using XNode;
using UnityEditor; // Required for AssetDatabase
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;
using plugpea;
using Unity.VisualScripting;
using UnityEngine.Windows;
using System.Linq;
using System.IO;
using XNodeEditor.Internal;
namespace plugpea
{
    public static class GraphUtils
    {
        public static PlugPeaGraph DuplicateGraph(PlugPeaGraph originalGraph, string savePath, GameObject gameobjects)
        {
            PlugPeaGraph newGraph = ScriptableObject.CreateInstance<PlugPeaGraph>();

            //find plugpea component 
            PlugPea pp = gameobjects.GetComponent<PlugPea>();
            // pp.Awake();
            newGraph.plugpeaID = pp.Id;
            newGraph.plugpeaGUID = pp.objectGUID;
            //newGraph.childIDs = originalGraph.childIDs;
            // Save the new graph as an asset
            AssetDatabase.CreateAsset(newGraph, savePath);

            Dictionary<Node, Node> nodeMapping = new Dictionary<Node, Node>();

            foreach (Node node in originalGraph.nodes)
            {

                Node newNode = newGraph.AddNode(node.GetType());

                newNode.name = node.name;
                newNode.graph = newGraph;
                foreach (FieldInfo field in node.GetType().GetFields(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Default))//
                {
                    if (field.FieldType.IsSubclassOf(typeof(NodePort)))
                    {
                        // skip NodePort fields, we'll handle these later
                        continue;
                    }
                    //Debug.Log("Duplicating field: " + field.Name);
                    field.SetValue(newNode, field.GetValue(node));
                }

                AssetDatabase.AddObjectToAsset(newNode, newGraph);
                nodeMapping.Add(node, newNode);
            }

            foreach (Node node in originalGraph.nodes)
            {
                Node newNode = nodeMapping[node];

                foreach (NodePort outputPort in node.Outputs)
                {
                    NodePort newOutputPort = newNode.GetOutputPort(outputPort.fieldName);

                    foreach (NodePort inputPort in outputPort.GetConnections())
                    {
                        Node newInputNode = nodeMapping[inputPort.node];
                        NodePort newInputPort = newInputNode.GetInputPort(inputPort.fieldName);

                        newOutputPort.Connect(newInputPort);
                    }
                }


                try
                {
                    NodePort p = newNode.Outputs.First();

                    newNode.GetValue(p);
                }
                catch { Debug.Log("couldn't update the nodes"); }
                newNode.graph = newGraph;
                //Debug.Log(newNode.graph.name);
            }

            // Don't forget to save the assets and refresh the AssetDatabase after making changes
            // AssetDatabase.SaveAssets();
            // AssetDatabase.Refresh();
            //AssetDatabase.CreateAsset(newGraph, path);
            AssetDatabase.SaveAssets();
            AssetDatabase.Refresh();
            Object savedGraph = AssetDatabase.LoadAssetAtPath<Object>(savePath);

            return newGraph;
        }
    }
}