using System.Collections.Generic;
using System.IO;
using UnityEngine;
namespace plugpea
{
    public class PersistantPrivateIds
    {
        private Dictionary<string, string> dictionary;
        private string filePath;

        public PersistantPrivateIds()
        {
            filePath = Path.Combine(Application.dataPath, "PrivateIds");

            if (File.Exists(filePath))
            {
                string json = File.ReadAllText(filePath);
                dictionary = JsonUtility.FromJson<Serialization<string, string>>(json).ToDictionary();
            }
            else
            {
                dictionary = new Dictionary<string, string>();
            }
        }

        public void Set(string key, string value)
        {
            dictionary[key] = value;
            Save();
        }

        public bool TryGet(string key, out string value)
        {
            return dictionary.TryGetValue(key, out value);
        }

        public void Remove(string key)
        {
            dictionary.Remove(key);
            Save();
        }

        public void Clear()
        {
            dictionary.Clear();
            Save();
        }

        private void Save()
        {
            var serialization = new Serialization<string, string>(dictionary);
            string json = JsonUtility.ToJson(serialization);
            File.WriteAllText(filePath, json);
        }
    }



    [System.Serializable]
    public class Serialization<K, V>
    {
        [SerializeField]
        List<K> keys;
        [SerializeField]
        List<V> values;

        public Serialization(Dictionary<K, V> dictionary)
        {
            keys = new List<K>(dictionary.Keys);
            values = new List<V>(dictionary.Values);
        }

        public Dictionary<K, V> ToDictionary()
        {
            var dictionary = new Dictionary<K, V>();
            for (int i = 0; i != Mathf.Min(keys.Count, values.Count); i++)
                dictionary.Add(keys[i], values[i]);
            return dictionary;
        }
    }
}