using System;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using XNodeEditor;
using Plane = PlugPeaNodes.Plane;
using plugpea;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Pick (Item)", 1)]
    [DontFold]
    public class ListItem : BaseNode
    {
        [Input] public object InputData;
        [Input(ShowBackingValue.Never)] public List<float> Indices;
        [Input] public bool Wrap;
        [Output] public List<object> Output;

        protected override void Init()
        {
            base.Init();

        }

        public override object GetValue(NodePort port)
        {
            // Get the input data
            object inputData = GetInputValue<object>("InputData");

            // Check if the inputData is not null
            if (inputData == null)
            {
                Debug.LogWarning("InputData is null.");
                return null;
            }

            // Get the indices
            List<float> indices = GetInputValue<List<float>>("Indices", this.Indices);
            bool wrap = GetInputValue<bool>("Wrap", this.Wrap);


            // Check if the indices is not null
            if (indices == null)
            {
                Debug.LogWarning("Indices are null.");
                return null;
            }

            // Get the type of inputData
            string inputDataType = inputData.GetType().ToString();



            switch (inputDataType)
            {

                case "System.Collections.Generic.List`1[UnityEngine.Vector3]":
                    return GetOutputList<Vector3>(inputData, indices, wrap);

                case "System.Collections.Generic.List`1[System.Single]":
                    return GetOutputList<float>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[System.Int32]":
                    return GetOutputList<int>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[PlugPeaNodes.Plane]":
                    return GetOutputList<Plane>(inputData, indices, wrap);

                case "System.Collections.Generic.List`1[Curve]":
                    return GetOutputList<Curve>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[UnityEngine.Mesh]":
                    return GetOutputList<Mesh>(inputData, indices, wrap);

                case "System.Collections.Generic.List`1[PlugPeaNodes.Curve]":
                    return GetOutputList<Curve>(inputData, indices, wrap);
                // Add other cases as required...
                default:
                    Debug.LogWarning($"Unsupported type {inputDataType} in ListItemNode.");
                    return null;
            }


        }


        private List<T> GetOutputList<T>(object inputData, List<float> indices, bool enableWrap)
        {
            List<T> inputList = inputData as List<T>;
            if (inputList == null)
            {
                Debug.LogWarning("Invalid input in ListItemNode.");
                return null;
            }

            List<T> outputList = new List<T>();
            int listCount = inputList.Count;
            foreach (var index in indices)
            {
                if (enableWrap)
                {
                    // Wrap the index if it's outside the range of the list
                    int wrappedIndex =(int) ((index % listCount) + listCount) % listCount;
                    // Add the element at the wrapped index to the output list
                    outputList.Add(inputList[wrappedIndex]);
                }
                else
                {
                    if (index >= 0 && index < listCount)
                    {
                        outputList.Add(inputList[(int)index]);
                    }
                }
            }
            return outputList;
        }
        [HideInInspector]
        public PlugPeaGraph currentGraph;
        [HideInInspector]
        public int id;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.UpdateNode(this);
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);
        }
    }






    [CustomNodeEditor(typeof(ListItem))]
    public class ListItemTestEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            ListItem node = target as ListItem;

            NodeEditorGUILayout.PortField(target.GetInputPort("InputData"));
            // NodeEditorGUILayout.PortField(target.GetInputPort("Indices"));
            //NodeEditorGUILayout.PortField(target.GetInputPort("Wrap") );
            NodeEditorGUILayout.PortField(target.GetOutputPort("Output"));
        }
    }


}