using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Numerics;
using Unity.VisualScripting;
using UnityEngine;
using XNode;
using XNodeEditor;
using Vector3 = UnityEngine.Vector3;
using plugpea;
namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/List Length", 4)]
    [DontFold]
    public class ListLength : BaseNode
    {
        [Input(ShowBackingValue.Never, ConnectionType.Override, connectionType = ConnectionType.Multiple)]
        public object List; // Input list of radii

        [Output] public List<float> Length;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();

            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
 
            currentGraph.UpdateNode(this);
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override string GetString()
        {
            return "Draw";
        }
        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override object GetValue(NodePort port)
        {
            var input = GetInputValue("List", this.List);

            List<float> ListCount = new List<float>() { 0f};
            if(input is List<object> ListofObjects)
            {
                ListCount[0] = (float)ListofObjects.Count;
                return ListCount;
            }
            else if (input is List<float> listoffloats)
            {

                ListCount[0] = (float)listoffloats.Count;
                return ListCount;
            }
            else if (input is List<Plane> PlaneListInput)
            {
                ListCount[0] = (float)PlaneListInput.Count;
                return ListCount;
            }
            else if (input is List<Curve> curveListInput)
            {
                ListCount[0] = (float)curveListInput.Count;
                return ListCount;
            }
            else if (input is List<Mesh> meshListInput)
            {
                ListCount[0] = (float)meshListInput.Count;
            }
            else if (input is List<Vector3> PointListInput)
            {
                ListCount[0] = (float)PointListInput.Count;
            }
            else if (input is List<int> intListCount)
            {
                ListCount[0] = (float)intListCount.Count;
            }
            else
            {
               
            }

            return ListCount;
        }
    }

    [CustomNodeEditor(typeof(ListLength))]
    public class ListLengthEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            ListLength node = target as ListLength;

            NodeEditorGUILayout.PortField(new GUIContent("List"), node.GetPort("List"));
        }
    }
}
