using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using XNode;
using static UnityEngine.GraphicsBuffer;
using static XNodeEditor.NodeEditor;
using XNodeEditor;
using static UnityEditor.Progress;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using Unity.VisualScripting;
using System.Linq;
using System.Collections;
using plugpea;
namespace PlugPeaNodes
{
    [NodeWidth(300)]
    [CreateNodeMenu("Utility/Panel", 4)]
    [DontFold]
    public class Panel : BaseNode
    {
        [Input(ShowBackingValue.Never)] public object InputData; // Can accept a List<float>, List<int>, or List<Vector3>
                                                                 // [Output] public object OutputData; // Can return a List<float>, List<int>, or List<Vector3>
        [HideInInspector]
        public List<string> textToDisplay = new List<string>();
        [HideInInspector]
        public string displayText = "Panel";
        GameObject currentGameobject;
        PlugPeaGraph currentGraph;
        [HideInInspector]
        public int id;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID() + this.graph.GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.UpdateNode((BaseNode)this);

        }

        public override string GetString()
        {
            return "Panel";
        }


        protected override void Init()
        {
            base.Init();
            var gg = (PlugPeaGraph)this.graph;



        }
        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            currentGraph.UpdateAndPropogate(this);

        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);

            currentGraph.UpdateAndPropogate(this);


        }
        public override object GetValue(NodePort port)
        {
            displayText = "Panel";
            var InputData = GetInputValue<object>("InputData", this.InputData);
            // var OutputData = GetInputValue<object>("OutputData", this.OutputData);

            textToDisplay.Clear();

            if (InputData != null)
            {
                if (InputData is List<float> floatList)
                {
                    displayText = floatList.Count + " floats";
                    for (int i = 0; i < floatList.Count; i++)
                    {
                        textToDisplay.Add(i + "-" + floatList[i].ToString());
                    }
                }
                else if (InputData is List<int> intList)
                {
                    displayText = intList.Count + " int";
                    for (int i = 0; i < intList.Count; i++)
                    {
                        textToDisplay.Add(i + "-" + intList[i].ToString());
                    }
                }
                else if (InputData is List<Vector3> vector3List)
                {
                    displayText = vector3List.Count + " Vector3";
                    for (int i = 0; i < vector3List.Count; i++)
                    {
                        textToDisplay.Add(i + "-" + vector3List[i].ToString());
                    }
                }

                else if (InputData is List<Plane> PlaneList)
                {
                    displayText = PlaneList.Count + " Planes";
                    for (int i = 0; i < PlaneList.Count; i++)
                    {
                        textToDisplay.Add($"{i}-O:{PlaneList[i].basePoint} Z: {PlaneList[i].Normal}");
                    }
                }

                else if (InputData is List<Mesh> MeshList)
                {
                    if (MeshList.Count == 1) { displayText = MeshList.Count + " Mesh"; }
                    else { displayText = MeshList.Count + " Meshes"; }

                    for (int i = 0; i < MeshList.Count; i++)
                    {
                        textToDisplay.Add($"Mesh V: {MeshList[i].vertexCount} F:{MeshList[i].triangles.Length}");
                    }
                }

                else if (InputData is List<Curve> CurveList)
                {
                    if (CurveList.Count == 1) { displayText = CurveList.Count + " Curve"; }
                    else { displayText = CurveList.Count + " Curves"; }
                    if (CurveList[0].type == 2)
                    {
                        for (int i = 0; i < CurveList.Count; i++)
                        {

                            textToDisplay.Add($"{i}-Circle (R: {CurveList[i].radius} , C:{CurveList[i].center.basePoint})");
                        }
                    }
                    if (CurveList[0].type == 0)
                    {
                        for (int i = 0; i < CurveList.Count; i++)
                        {
                            textToDisplay.Add($"{i}-Polyline");
                        }
                    }
                    if (CurveList[0].type == 1)
                    {
                        for (int i = 0; i < CurveList.Count; i++)
                        {
                            textToDisplay.Add($"{i}-Curve");
                        }
                    }
                }
            }

            // Return value for OutputData. Modify as needed.
            return null;
        }



    }


    [CustomNodeEditor(typeof(Panel))]
    public class ExampleNodeEditor : NodeEditor
    {
        private Vector2 scrollPosition;
        private const float ITEM_HEIGHT = 20f; // Adjust as needed
        private const float MAX_HEIGHT = 200f; // Adjust as needed
        Panel node;
        public override void OnHeaderGUI()
        {
            Panel bb = (Panel)target;

            //EditorGUILayout.LabelField(node.displayText);
            // string previousName = target.name;
            //target.name = node.displayText;
            //base.OnHeaderGUI();
            // target.name = previousName;
            GUILayout.Label(bb.displayText, NodeEditorResources.styles.nodeHeader, GUILayout.Height(30));
        }

        public override void OnBodyGUI()
        {
            base.OnBodyGUI();

            Panel node = target as Panel;

            NodeEditorGUILayout.PortField(target.GetInputPort("InputData"));
            // NodeEditorGUILayout.PortField(target.GetOutputPort("OutputData"));
            string longestLine = node.textToDisplay.OrderByDescending(s => s.Length).FirstOrDefault();

            // You can adjust the GUIStyle here to match your actual text style

            if (node != null && node.textToDisplay != null)
            {
                float totalHeight = node.textToDisplay.Count * ITEM_HEIGHT;
                if (totalHeight > MAX_HEIGHT)
                {
                    scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition, GUILayout.Height(MAX_HEIGHT));

                    foreach (var item in node.textToDisplay)
                    {
                        EditorGUILayout.LabelField(item);
                    }

                    EditorGUILayout.EndScrollView();
                }
                else
                {
                    foreach (var item in node.textToDisplay)
                    {
                        EditorGUILayout.LabelField(item);
                    }
                }
            }
            EditorApplication.delayCall += () => NodeEditorWindow.current.Repaint();
        }
    }



    //[CustomNodeEditor(typeof(ExampleNode))]
    //public class ExampleNodeEditor : NodeEditor
    //{
    //    private Vector2 scrollPosition;

    //    public override void OnBodyGUI()
    //    {
    //        base.OnBodyGUI();
    //        ExampleNode node = target as ExampleNode;

    //        NodeEditorGUILayout.PortField(target.GetInputPort("InputData"));
    //        NodeEditorGUILayout.PortField(target.GetOutputPort("OutputData"));

    //        if (node != null)
    //        {
    //            if (node.textToDisplay != null)
    //            {
    //                scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition, GUILayout.Height(150));

    //                foreach (var item in node.textToDisplay)
    //                {
    //                    EditorGUILayout.LabelField(item);
    //                }
    //                EditorGUILayout.EndScrollView();
    //            }
    //        }
    //    }
    //}
}