using System;
using System.Collections.Generic;
using UnityEngine;
using XNode;

using XNodeEditor;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using plugpea;
namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Remove (Item)", 1)]
    [DontFold]
   

    public class RemoveItem : BaseNode
    {
        [Input] public object InputData;
        [Input(ShowBackingValue.Never)] public List<int> Indices;
        [Input] public bool Wrap;
        [Output] public object Output;

        protected override void Init()
        {
            base.Init();
        }

        public override object GetValue(NodePort port)
        {
            // Get the input data
            object inputData = GetInputValue<object>("InputData");

            // Check if the inputData is not null
            if (inputData == null)
            {
                Debug.LogWarning("InputData is null.");
                return null;
            }

            // Get the indices
            List<int> indices = GetInputValue<List<int>>("Indices");

            // Check if the indices are not null
            if (indices == null)
            {
                Debug.LogWarning("Indices are null.");
                return null;
            }

            // Get the wrap value
            bool wrap = GetInputValue<bool>("Wrap");

            // Get the type of inputData
            string inputDataType = inputData.GetType().ToString();

            switch (inputDataType)
            {
                case "System.Collections.Generic.List`1[System.Single]":
                    return RemoveItemsFromList<float>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[System.Int32]":
                    return RemoveItemsFromList<int>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[Plane]":
                    return RemoveItemsFromList<Plane>(inputData, indices, wrap);
                case "System.Collections.Generic.List`1[PlugPeaNodes.Curve]":
                    return RemoveItemsFromList<Curve>(inputData, indices, wrap);
                // Add other cases as required...
                default:
                    Debug.LogWarning($"Unsupported type {inputDataType} in RemoveItemNode.");
                    return null;
            }
        }

        private List<T> RemoveItemsFromList<T>(object inputData, List<int> indices, bool wrap)
        {
            List<T> inputList = inputData as List<T>;
            if (inputList == null)
            {
                Debug.LogWarning("Invalid input in RemoveItemNode.");
                return null;
            }

            int listCount = inputList.Count;

            // Sort the indices in descending order
            indices.Sort((x, y) => y.CompareTo(x));

            foreach (int index in indices)
            {
                if (wrap)
                {
                    // Wrap the index if it's outside the range of the list
                    int wrappedIndex = ((index % listCount) + listCount) % listCount;
                    inputList.RemoveAt(wrappedIndex);
                }
                else if (index >= 0 && index < listCount)
                {
                    inputList.RemoveAt(index);
                }
            }

            return inputList;
        }
    }

    [CustomNodeEditor(typeof(RemoveItem))]
    public class RemoveItemEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            RemoveItem node = target as RemoveItem;

            NodeEditorGUILayout.PortField(target.GetInputPort("InputData"));
   
            NodeEditorGUILayout.PortField(target.GetOutputPort("Output"));
        }
    }

}