using UnityEditor;
using UnityEngine;
using XNode;
using XNodeEditor;
using static XNode.Node;

namespace PlugPeaNodes
{

    [CustomNodeEditor(typeof(Slider))]
    public class SliderEditor : NodeEditor
    {
        // Reference to the GUIStyle used for the slider
        private GUIStyle sliderStyle;


        public override int GetWidth()
        {
            return 200;
        }
        public override void OnHeaderGUI()
        {
            Slider sliderNode = (Slider)target;
            if (sliderNode.Lock)
            {
                GUILayout.Label(sliderNode.name, NodeEditorResources.styles.nodeHeader, GUILayout.Height(30));
            }
            else
            {
                //base.OnHeaderGUI();
                GUILayout.BeginHorizontal();


                GUILayout.Label(sliderNode.name, NodeEditorResources.styles.nodeHeader, GUILayout.Height(30));
                if (sliderNode.integer)
                {
                    GUILayout.Label(sliderNode.sliderValue.ToString("0"), NodeEditorResources.styles.nodeHeader, GUILayout.Height(30));
                }
                else
                {
                    GUILayout.Label(sliderNode.sliderValue.ToString("0.00"), NodeEditorResources.styles.nodeHeader, GUILayout.Height(30));
                }


                GUILayout.EndHorizontal();
            }
        }
        public override void OnBodyGUI()
        {
            // Retrieve the target node
            Slider sliderNode = (Slider)target;
            GUILayout.BeginHorizontal();
            if (sliderNode.Lock)
            {
                sliderNode.Lock = GUILayout.Toggle(sliderNode.Lock, "Unlock");
            }
            else
            {
                sliderNode.Lock = GUILayout.Toggle(sliderNode.Lock, "Lock");
            }
            if (sliderNode.integer)
            {
                sliderNode.integer = GUILayout.Toggle(sliderNode.integer, "Float");
            }
            else
            {
                sliderNode.integer = GUILayout.Toggle(sliderNode.integer, "Integer");
            }
            GUILayout.EndHorizontal();
            // Create a GUIStyle for the slider
            if (sliderStyle == null)
            {
                sliderStyle = new GUIStyle(GUI.skin.horizontalSlider);
                sliderStyle.fixedHeight = 20f;
                sliderStyle.fontSize = 14;
            }


            GUIStyle boldLabelStyle = new GUIStyle(GUI.skin.label);

            // Draw the slider UI
            if (sliderNode.Lock == false)
            {
                boldLabelStyle.fontStyle = FontStyle.Bold;
                boldLabelStyle.fontSize = 20;
                GUILayout.Space(10f);

                GUILayout.BeginHorizontal();
                if (sliderNode.integer)
                {
                    sliderNode.sliderValue = GUILayout.HorizontalSlider((int)sliderNode.sliderValue, (int)sliderNode.minValue, (int)sliderNode.maxValue, GUILayout.Height(20f), GUILayout.ExpandWidth(true));
                }
                else
                {
                    sliderNode.sliderValue = GUILayout.HorizontalSlider(sliderNode.sliderValue, sliderNode.minValue, sliderNode.maxValue, GUILayout.Height(20f), GUILayout.ExpandWidth(true));
                }
                GUILayout.Space(-300f);
                // Display the input port
                //NodeEditorGUILayout.PortField(new GUIContent("Input"), target.GetInputPort("inputValue"), GUILayout.Width(100f));

                // Display the output port
                NodeEditorGUILayout.PortField(new GUIContent(""), target.GetOutputPort("outputValue"));

                GUILayout.EndHorizontal();
                GUILayout.Space(-5f);
            }
            else
            {
                boldLabelStyle.fontStyle = FontStyle.Bold;
                boldLabelStyle.fontSize = 30;
                //GUILayout.Space(-10f);
                GUILayout.BeginHorizontal();

                //  GUILayout.FlexibleSpace();
                if (sliderNode.integer)
                {
                    GUILayout.Label(sliderNode.intSliderValue.ToString("0"), boldLabelStyle, GUILayout.Height(50));
                }
                else
                {
                    GUILayout.Label(sliderNode.sliderValue.ToString("0.00"), boldLabelStyle, GUILayout.Height(50));
                }



                GUILayout.Space(-200f);
                NodeEditorGUILayout.PortField(new GUIContent(""), target.GetOutputPort("outputValue"));
                GUILayout.EndHorizontal();
            }
        }
    }
}