using UnityEngine;
using XNode;
using System.Collections.Generic;
using PlugPeaNodes;
using XNodeEditor;
using static UnityEditor.GenericMenu;
using Plane = PlugPeaNodes.Plane;
using Unity.VisualScripting;
using UnityEngine.XR;
using System.Linq;
using System.Numerics;
using Quaternion = UnityEngine.Quaternion;
using Vector3 = UnityEngine.Vector3;
using Curve = PlugPeaNodes.Curve;
using System.Security.Cryptography;
using plugpea;
namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Rotate (Axis)", 1)]
    [DontFold]
    public class RotateAround : BaseNode
    {
        [Input(ShowBackingValue.Never)]
        public object GeometryIn;

        [Input(ShowBackingValue.Never)]
        public List<Vector3> Axis;

        [Input(ShowBackingValue.Never)]
        public List<Vector3> Center;

        [Input(ShowBackingValue.Never)]
        public List<float> Angle;

        [Output]
        public object GeometryOut;
        public bool preview;
        // Override the Value method to process the input object
        protected override void Init()
        {

            base.Init();
            Axis = new List<Vector3>() { new Vector3(0, 1, 0) };
            Center = new List<Vector3>() { Vector3.zero };
            Angle = new List<float>() { 45f };
        }

        public override string GetString()
        {
            string name = "Draw";
            if (preview == false) { name = null; }
            return name;
        }
        public override object GetValue(NodePort port)
        {

            object[] geometryIns = GetInputValues("GeometryIn", GeometryIn);
            List<Vector3>[] Axis = GetInputValues("Axis", this.Axis);
            List<Vector3>[] Center = GetInputValues("Center", this.Center);
            List<float>[] Angle = GetInputValues("Angle", this.Angle);
            Vector3 axis = new Vector3(0, 1, 0);
            Vector3 center = new Vector3(0, 0, 0);


            List<Vector3> axises = new List<Vector3>() { };
            List<Vector3> centers = new List<Vector3>() { };
            List<float> angles = new List<float>() { };


            int longestList = 0;
            if (Axis != null)
            {

                for (int i = 0; i < Axis.Length; i++)
                {
                    for (int j = 0; j < Axis[i].Count; j++)
                    {
                        axises.Add(Axis[i][j]);
                    }
                }

                if (axises.Count > longestList)
                {
                    longestList = axises.Count;
                }
            }
            else
            {
                axises = new List<Vector3>() { new Vector3(0, 1, 0) };
            }

            if (Center != null)
            {
                for (int i = 0; i < Center.Length; i++)
                {
                    for (int j = 0; j < Center[i].Count; j++)
                    {
                        centers.Add(Center[i][j]);
                    }
                }
                if (centers.Count > longestList)
                {
                    longestList = centers.Count;
                }
            }
            else
            {
                centers = new List<Vector3>() { new Vector3(0, 0, 0) };
            }
            if (Angle != null)
            {
                for (int i = 0; i < Angle.Length; i++)
                {
                    for (int j = 0; j < Angle[i].Count; j++)
                    {
                        angles.Add(Angle[i][j]);
                    }
                }
                if (angles.Count > longestList)
                {
                    longestList = angles.Count;
                }
            }
            else
            {
                angles = new List<float>() { 90f };
            }
            while (angles.Count < longestList)
            {
                angles.Add(angles[angles.Count - 1]);
            }
            while (centers.Count < longestList)
            {
                centers.Add(centers[centers.Count - 1]);
            }
            while (axises.Count < longestList)
            {
                axises.Add(axises[axises.Count - 1]);       
            }


            int typeIndex = 0; //0 = mesh, 1 = curve, 2 = Vector3 , 3 = Plane

            List<Mesh> geometryMeshout = new List<Mesh>();
            List<Curve> geometryCurveOut = new List<Curve>();
            List<Vector3> geometryVector3Out = new List<Vector3>();
            List<Plane> geometryPlaneOut = new List<Plane>();
            List<float> angg = new List<float>();

            List<Curve> anggs = new List<Curve>();


            for(int i = 0; i < longestList; i++)
            {
                geometryCurveOut.Add(CreateInstance<Curve>());
            }
            foreach (var geometryInn in geometryIns)
            {
               
                if (geometryInn is List<Mesh>)
                {
                    List<Mesh> meshes = (List<Mesh>)geometryInn;



                    for (int i = 0; i < longestList; i++)
                    {
                        Mesh mesh = new Mesh();
                        try
                        {
                            mesh.vertices = meshes[i].vertices;
                            mesh.triangles = meshes[i].triangles;
                        }
                        catch
                        {
                            mesh.vertices = meshes[meshes.Count - 1].vertices;
                            mesh.triangles = meshes[meshes.Count - 1].triangles;
                        }
                        Quaternion rotation = Quaternion.AngleAxis(angles[i], axises[i]);

                        Vector3[] vertices = mesh.vertices;

                        for (int j = 0; j < vertices.Length; j++)
                        {
                            vertices[j] = rotation * (vertices[j] - centers[i]) + centers[i];
                        }

                        mesh.vertices = vertices;
                        mesh.RecalculateNormals();
                        mesh.RecalculateBounds();

                        // rotatedMeshes.Add(mesh);
                        geometryMeshout.Add(mesh);
                    }

                    
                }
                //else if (geometryInn is List<Curve>)
                //{
                //    typeIndex = 1;
                //    List<Curve> curves = (List<Curve>)geometryInn;
                
                //    Debug.Log("count" + curves.Count);
                //    for (int i = 0 ; i < longestList; i++)
                //    {
                            
                //         Curve curve = CreateInstance<Curve>();
                //        //Curve curve = new Curve();
                //        Vector3[] controlPoints ; 
                //        try
                //        {
                //            controlPoints = curves[i].ControlPoints;
                //            //curve.ControlPoints = curves[i].ControlPoints;
                            
                //            curve.type = curves[i].type;
                //            if (curves[i].type ==2)
                //            {
                                
                //                curve.center = curves[i].center;
                //                curve.radius = curves[i].radius;
                //            }
                //        }
                //        catch
                //        {
                           
                //            controlPoints  = curves[curves.Count - 1].ControlPoints;
                //            //curve.ControlPoints = curves[curves.Count - 1].ControlPoints;
                //            curve.type = curves[curves.Count - 1].type;
                //            if (curves[curves.Count - 1].type == 2)
                //            {
                               
                //                curve.center = curves[curves.Count - 1].center;
                //                curve.radius = curves[curves.Count - 1].radius;
                //            }
                //        }
                        
                //        Quaternion rotation = Quaternion.AngleAxis(angles[i], axises[i]);
                        


                //        for (int j = 0; j < controlPoints.Length; j++)
                //        {
                //            if (controlPoints[j] != null)
                //            {
                //                controlPoints[j] = rotation * (controlPoints[j] - centers[i]) + centers[i];
                //            }
                //        }
                //        curve.ControlPoints = controlPoints;
                   
                //        if (curve.type == 2)
                //        {

                //            curve.center.basePoint = rotation * (curve.center.basePoint - centers[i]) + centers[i];
                        
                //            curve.center.X = (rotation * curve.center.X).normalized;
                //            curve.center.Y = (rotation * curve.center.Y).normalized;
                //            curve.center.Normal = (rotation * curve.center.Normal).normalized;
                //        }

                //        geometryCurveOut[i] = curve;
                //        Debug.Log("angle" + angles[i]);
                //        //geometryCurveOut.Add(curve);
                //        angg.Add(angles[i]);
                //        //Debug.Log(curve.center.basePoint);
                //        anggs.Add(curve);
                //    }
                //    Debug.Log("hey");


                //    Debug.Log(angg[0]);
                //    Debug.Log(angg[1]);
                //    Debug.Log(angg[2]);

                //}
               
                else if (geometryInn is List<Vector3>)
                {
                    typeIndex = 2;
                    List<Vector3> points = (List<Vector3>)geometryInn;
                    for (int i = 0; i < longestList; i++)
                    {
                        Vector3 point;
                        try
                        {
                            point = points[i];
                        }
                        catch
                        {
                            point = points[points.Count - 1];
                        }
                        Quaternion rotation = Quaternion.AngleAxis(angles[i], axises[i]);
                        points[i] = rotation * (points[i] - centers[i]) + centers[i];

                        geometryVector3Out.Add(points[i]);
                    }
                }

                else if (geometryInn is List<Curve>)
                {
                    typeIndex = 3;
                    List<Curve> planes = (List<Curve>)geometryInn;
                    for (int i = 0; i < longestList; i++)
                    {
                        Curve cs = CreateInstance<Curve>();
                        try
                        {
                            cs.center.basePoint = planes[i].center.basePoint;
                            cs.center.X = planes[i].center.X;
                            cs.center.Y = planes[i].center.Y;
                            cs.center.Normal = planes[i].center.Normal;
                        }
                        catch
                        {
                            cs.ControlPoints = planes[planes.Count - 1].ControlPoints;
                            cs.center.basePoint = planes[planes.Count - 1].center.basePoint;
                            cs.center.X = planes[planes.Count - 1].center.X;
                            cs.center.Y = planes[planes.Count - 1].center.Y;
                            cs.center.Normal = planes[planes.Count - 1].center.Normal;
                        }
                        Quaternion rotation = Quaternion.AngleAxis(angles[i], axises[i]);

                        cs.center.basePoint = rotation * (cs.center.basePoint - centers[i]) + centers[i];

                        cs.center.X = (rotation * (cs.center.X) ).normalized;
                        cs.center.Y = (rotation * (cs.center.Y )).normalized;
                        cs.center.Normal = (rotation * (cs.center.Normal ) ).normalized;
                        for (int k = 0; k < cs.ControlPoints.Length; k++)
                            cs.ControlPoints[k] = rotation * (cs.ControlPoints[k] - centers[i]) + centers[i];
                        geometryCurveOut.Add(cs);
                    }
                }
                else if (geometryInn is List<Plane>)
                {
                    typeIndex = 3;
                    List<Plane> planes = (List<Plane>)geometryInn;
                    for (int i = 0; i < longestList; i++)
                    {
                        Plane plane = CreateInstance<Plane>();
                        try
                        {
                            plane.basePoint = planes[i].basePoint;
                            plane.X = planes[i].X;
                            plane.Y = planes[i].Y;
                            plane.Normal = planes[i].Normal;
                        }
                        catch
                        {

                            plane.basePoint = planes[planes.Count - 1].basePoint;
                            plane.X = planes[planes.Count - 1].X;
                            plane.Y = planes[planes.Count - 1].Y;
                            plane.Normal = planes[planes.Count - 1].Normal;
                        }
                        Quaternion rotation = Quaternion.AngleAxis(angles[i], axises[i]);

                        plane.basePoint = rotation * (plane.basePoint - centers[i]) + centers[i];

                        plane.X = (rotation * (plane.X)).normalized;
                        plane.Y = (rotation * (plane.Y)).normalized;
                        plane.Normal = (rotation * (plane.Normal)).normalized;

                        geometryPlaneOut.Add(plane);
                    }
                }
                else
                {
                    Debug.LogWarning("Rotate Around node does not support the provided geometry type.");

                }

                //Debug.Log(geometryCurveOut[0].center.basePoint);

            }

            if (typeIndex == 0)
            {
                return geometryMeshout;
            }
            else if (typeIndex == 1)
            {
                 return geometryCurveOut;
                //return anggs;
            }
            else if (typeIndex == 2)
            {
                return geometryVector3Out;
            }
            else if (typeIndex == 3)
            {
                return geometryPlaneOut;
            }
            else
            {
                return null;
            }

        }




    }







[CustomNodeEditor(typeof(RotateAround))]
public class RotateEditor : NodeEditor
{
    public override void OnBodyGUI()
    {
        base.OnBodyGUI();
        RotateAround node = target as RotateAround;

        NodeEditorGUILayout.PortField(new GUIContent("Geometry"), node.GetPort("GeometryIn"));
        NodeEditorGUILayout.PortField(new GUIContent("Geometry"), node.GetPort("GeometryOut"));
    }
}
}