using System.Collections;
using System.Collections.Generic;
using System.Numerics;
using UnityEngine;
using XNode;
using Vector3 = UnityEngine.Vector3;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using PlugPeaNodes;
namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Construct (Plane)", 3)]
    [DontFold]
    public class ConstructPlane : BaseNode
    {
        // Start is called before the first frame update
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<Vector3> Points;
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<Vector3> X;
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<Vector3> Y;

        bool preview = true;
        [Output] public List<Plane> Planes;

        // PlugPea1 gg =(PlugPea1)Component.FindObjectOfType<PlugPea1>();
        [HideInInspector]
        public string previewName = "Preview On";

        protected override void Init()
        {
            //Create a GUIStyle for the slider
            base.Init();
            Points = new List<Vector3>() { new Vector3(0, 0, 0) };
            X = new List<Vector3>() { new Vector3(1, 0, 0) };
            Y = new List<Vector3>() { new Vector3(0, 1, 0) };
        }
        int id;

        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            //var pp = currentGraph.getPlugPeaViaGUID();
            //if (pp != null && !pp.planeDictPreviewBool.ContainsKey(id))
            //{
            //    pp.planeDictPreviewBool.Add(id, true);
            //}
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }
        public override string GetString()
        {
            return "Draw";
        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.planeDictPreview.ContainsKey(id)) { pp.planeDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override object GetValue(NodePort port)
        {
            PlugPeaGraph graph = (PlugPeaGraph)this.graph;



            var Points = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("Points", this.Points));
            var X = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("X", this.X));
            var Y = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("Y", this.Y));
            if (Points == null)
            {
                Points = new List<Vector3>() { new Vector3(0, 0, 0) };
            }
            if (X == null)
            {
                X = new List<Vector3>() { new Vector3(1, 0, 0) };
            }
            if (Y == null)
            {
                Y = new List<Vector3>() { new Vector3(0, 1, 0) };
            }


            if (Points == null || X == null || Y == null) { return null; }
            int maxLength = Mathf.Max(Points.Count, X.Count, Y.Count);
            while (Points.Count < maxLength)
            {
                Points.Add(Points[Points.Count - 1]);
            }
            while (X.Count < maxLength)
            {
                X.Add(X[X.Count - 1]);
            }
            while (Y.Count < maxLength)
            {
                Y.Add(Y[Y.Count - 1]);
            }
            List<Plane> planes = new List<Plane>();
            for (int i = 0; i < maxLength; i++)
            {
                Plane testPlane = CreateInstance<Plane>();
                testPlane.basePoint = Points[i];
                testPlane.X = X[i].normalized;
                testPlane.Y = Y[i].normalized;
                testPlane.Normal = Vector3.Cross(testPlane.X, testPlane.Y);

                planes.Add(testPlane);
            }

            NodeGraph gg = this.graph;

            return planes;
        }

        [HideInInspector]

        public void OnClickButton()
        {

            PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            //if (preview == false)
            //{
            //    preview = true;
            //    previewName = "Preview On";
            //    PlugPea[] components = FindObjectsOfType<PlugPea>();
            //    for (int i = 0; i < components.Length; i++)
            //    {
            //        PlugPea targetComponent = components[i];
            //        if (targetComponent.Id == currentGraph.plugpeaID)
            //        {
            //            var dict = targetComponent.planeDictPreviewBool;
            //            if (!dict.ContainsKey(id))
            //            {
            //                dict.Add(id, preview);
            //                currentGraph.UpdateNode(this);
            //            }
            //            else
            //            {
            //                dict[id] = preview;
            //            }
            //        }
            //    }
            //}
            //else
            //{
            //    preview = false;
            //    previewName = "Preview Off";
            //    PlugPea[] components = FindObjectsOfType<PlugPea>();
            //    for (int i = 0; i < components.Length; i++)
            //    {
            //        PlugPea targetComponent = components[i];
            //        if (targetComponent.Id == currentGraph.plugpeaID)
            //        {
            //            var dict = targetComponent.planeDictPreviewBool;
            //            if (!dict.ContainsKey(id))
            //            {
            //                dict.Add(id, preview);
            //                currentGraph.UpdateNode(this);
            //            }
            //            else
            //            {
            //                dict[id] = preview;
            //            }
            //        }
            //    }

            //}



        }


    }

    [CustomNodeEditor(typeof(ConstructPlane))]
    public class ConstructPlaneEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            ConstructPlane cc = (ConstructPlane)target;


            if (GUILayout.Button(cc.previewName))
            {
                cc.OnClickButton();
            }
        }
    }
}