using System.Collections;
using System.Collections.Generic;
using System.Numerics;
using UnityEngine;
using XNode;
using Vector3 = UnityEngine.Vector3;
using Plane = PlugPeaNodes.Plane;
using PlugPeaNodes;
using plugpea;
using XNodeEditor;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/XZ (Plane)", 3)]
    [DontFold]
    public class XZPlane : BaseNode
    {
        // Start is called before the first frame update
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<Vector3> Points;



        [Output] public List<Plane> Planes;

        bool preview;





        protected override void Init()
        {
            base.Init();
            Points = new List<Vector3>() { new Vector3(0, 0, 0) };
        }

        int id;

        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            //var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
            //if (pp != null && !pp.planeDictPreviewBool.ContainsKey(id))
            //{
            //    pp.planeDictPreviewBool.Add(id, true);
            //}
         
        }
        public override string GetString()
        {
            return "Draw";
        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.planeDictPreview.ContainsKey(id)) { pp.planeDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);
        }

        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {

            PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);


        }
        public override object GetValue(NodePort port)
        {

            var Points = GetInputValue<List<Vector3>>("Points", this.Points);



            int maxLength = Points.Count;


            List<Plane> planes = new List<Plane>();
            for (int i = 0; i < maxLength; i++)
            {
                Plane testPlane = CreateInstance<Plane>();

                testPlane.basePoint = Points[i];
                testPlane.X = new Vector3(1, 0, 0);
                testPlane.Y = new Vector3(0, 0, 1);
                testPlane.Normal = Vector3.Cross(testPlane.X, testPlane.Y);

                planes.Add(testPlane);
            }



            return planes;
        }


    }
    [CustomNodeEditor(typeof(XZPlane))]
    public class XZPlaneTypeNodeEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            XZPlane node = target as XZPlane;

            if (GUILayout.Button(node.previewName))
            {

                node.OnClickButton();

            }
        }
    }

}

