using plugpea;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using XNodeEditor;
namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Construct (Point)", 3)]
    [DontFold]
    public class ConstructPoint : BaseNode
    {
        [SerializeField]
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<float> x;
        [SerializeField]
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<float> y;
        [SerializeField]
        [Input(ShowBackingValue.Never, ConnectionType.Override)] public List<float> z;
        [SerializeField]
        [Output] public List<Vector3> pt = new List<Vector3>();
        bool preview = true;


        int id;
        PlugPea pp;
        string graphName;
        GameObject currentGameobject;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.pointDictPreview.ContainsKey(id)) { pp.pointDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);
        }


        [HideInInspector]
        public string previewName = "Preview On";

        protected override void Init()
        {
            base.Init();
        }

        public override object GetValue(NodePort port)
        {
            var gg = (PlugPeaGraph)this.graph;

            x = new List<float>() { 0f };
            y = new List<float>() { 0f };
            z = new List<float>() { 0f };
            x = GetInputValue<List<float>>("x", this.x);
            y = GetInputValue<List<float>>("y", this.y);
            z = GetInputValue<List<float>>("z", this.z);
            int maxLength = utils.GetLongestListLength(x, y, z);

            List<Vector3> pts = new List<Vector3>();

            for (int i = 0; i < maxLength; i++)
            {
                pts.Add(new Vector3(x[i], y[i], z[i]));
            }

            return pts;

        }
        public override string GetString()
        {
            return "Draw";

        }

        public void OnClickButton()
        {


            PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);

        }

    }


    [CustomNodeEditor(typeof(ConstructPoint))]
    public class pointEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            ConstructPoint cc = (ConstructPoint)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {

                cc.OnClickButton();


            }
        }

    }
}