using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;


namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Circle (Plane Rad.)", 4)]
    [DontFold]
    public class Circle : BaseNode
    {

        [Input(ShowBackingValue.Never, ConnectionType.Override)]
        public List<Plane> Plane; // Input list of planes


        [Input(ShowBackingValue.Never, ConnectionType.Override, connectionType = ConnectionType.Multiple)]
        public List<float> Radius; // Input list of radii


        [Output]
        public List<Curve> outputCircles; // Output list of circles
        [HideInInspector]
        bool preview ;

        int id;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();

            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }

        protected override void Init()
        {
            base.Init();
            List<Curve> circles = new List<Curve>();
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);

        }

        public override string GetString()
        {
            return "Draw";
        }

        public override object GetValue(NodePort port)
        {
           // Debug.Log(preview);
            // Debug.Log(currentGraph.childIDs.Count);effr
            //currentGameobject = gg.currentGameobject;

            List<Plane> planes = new List<Plane>();
            List<float> radii = new List<float>();
            List<Curve> circles = new List<Curve>();
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);

            Plane = new List<Plane>() { defaultPlane };
            Radius = new List<float>() { 5f };
            // Get the input plane and radius lists


            radii = utils.InsetMultiConnections(GetInputValues<List<float>>("Radius", Radius));

            planes = utils.InsetMultiConnections(GetInputValues<List<Plane>>("Plane", Plane));

            if (radii == null || planes == null) { return null; }
            int longestList = Mathf.Max(planes.Count, radii.Count); // Find the longest list length



            //Fill in the missing elements in input lists by repeating the last element
            while (planes.Count < longestList)
            {
                planes.Add(planes[planes.Count - 1]);
            }
            while (radii.Count < longestList)
            {
                radii.Add(radii[radii.Count - 1]);
            }

            circles.Clear();

            // Generate points on circles and create Curve instances
            for (int i = 0; i < longestList; i++)
            {
                Vector3[] pts = Curves.GeneratePointsOnCircle(planes[i], radii[i], 50);
                Curve circle = CreateInstance<Curve>();
                circle.radius = radii[i];
                circle.center = planes[i];
                circle.ControlPoints = pts;
                circle.type = 2;

                circles.Add(circle);
            }

            // Remove the extra circle dictionary entries if the longest list size has decreased


            // Return the output circles list
            if (port.fieldName == "outputCircles")
            {
                return circles;
            }
            else
            {
                return null;
            }

        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.curveDictPreview.ContainsKey(id)) { pp.curveDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        //ef
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);

        }
    }//efef

    [CustomNodeEditor(typeof(Circle))]
    public class circleEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            Circle cc = (Circle)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");wwess
                cc.OnClickButton();

            }
        }

    }


}