using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using plugpea;
using XNodeEditor;
using JetBrains.Annotations;
using UnityEditor.Experimental.GraphView;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Evaluate (Curve)", 4)]
    [DontFold]
    public class EvaluateCurve : BaseNode
    {

        [Input(ShowBackingValue.Never)] public List<Curve> curve;

        [Input(ShowBackingValue.Never)] public List<float> T;

        [Output] public List<Plane> plane;
        int id;
        PlugPea pp;
        string graphName;
        GameObject currentGameobject;

        bool preview ;

        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID() ;
            currentGraph = (PlugPeaGraph)this.graph;

           
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);

        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            pp = gg.getPlugPeaViaGUID();
            pp.curveDictPreview.Remove(id);
   
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);


            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);
        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;

          
            gg.UpdateAndPropogate(this);

        }
        [HideInInspector]
        public string previewName = "Preview On";



        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }
        protected override void Init()
        {
            base.Init();
            T = new List<float>() { 0f };
            preview = true;

        }

        public override object GetValue(NodePort port)
        {
            var gg = (PlugPeaGraph)this.graph;

            var t = utils.InsetMultiConnections(GetInputValues<List<float>>("T", T));
            var curvee = utils.InsetMultiConnections(GetInputValues<List<Curve>>("curve", this.curve));

       
            //find the longest list 
            int longestList = 0;

            if (curvee == null || t==null)
            {
                return null;
            }
            if (t.Count > longestList)
            {
                longestList = t.Count;
            }
            if (curvee.Count > longestList)
            {
                longestList = curvee.Count;
            }
            if (t.Count != 0 && t != null)
            {
                while (t.Count < longestList)
                {
                    t.Add(t[t.Count - 1]);
                }
            }
            if (curvee.Count != 0 && curvee != null)
            {
                while (curvee.Count < longestList)
                {
                    curvee.Add(curvee[curvee.Count - 1]);
                }
            }
            if(t==null || curvee == null ||t.Count==0 || curvee.Count ==0) { return null; }
            List<Plane> planes = new List<Plane>();
            for (int i = 0; i < longestList; i++)
            {
                Curve tempCurve = CreateInstance<Curve>();
                tempCurve.ControlPoints = curvee[i].ControlPoints;  
                tempCurve.type = curvee[i].type;
                if (tempCurve.type == 2)
                {
                    tempCurve.radius = curvee[i].radius;
                    tempCurve.center = curvee[i].center;
                }


                Vector3 pos = Curves.CurveInterpolation.GetPointOnCurve(tempCurve, tempCurve.type, (float)t[i]);
                Vector3 tangent = Curves.CurveInterpolation.GetTangentOnCurve(tempCurve, tempCurve.type, t[i]);
                Vector3 x;
                Vector3 y;

                GetPerpendicularVectors(tangent, out x, out y);
                Plane tempPlane = CreateInstance<Plane>();
                tempPlane.basePoint = pos;
                tempPlane.X = x;
                tempPlane.Y = y;
                tempPlane.Normal = tangent;
                planes.Add(tempPlane);



            }


            return planes;
        }



        public override string GetString()
        {

            string name = "Draw";
            return name;
        }




        public static void GetPerpendicularVectors(Vector3 normal, out Vector3 vector1, out Vector3 vector2)
        {
            // Find a vector that is not parallel to the normal vector
            Vector3 referenceVector;
            if (Mathf.Abs(normal.x) > Mathf.Abs(normal.y))
                referenceVector = new Vector3(normal.z, 0f, -normal.x);
            else
                referenceVector = new Vector3(0f, -normal.z, normal.y);

            // Calculate the first perpendicular vector
            vector1 = Vector3.Cross(normal, referenceVector).normalized;

            // Calculate the second perpendicular vector
            vector2 = Vector3.Cross(normal, vector1).normalized;
        }

    }
    [CustomNodeEditor(typeof(EvaluateCurve))]
    public class EvaluateCurveEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {

            EvaluateCurve cc = (EvaluateCurve)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                cc.OnClickButton();
            }

        }

    }
}