using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Line AB (Point)", 4)]
    [DontFold]
    public class LineAB : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Vector3> PointA;
        [Input(ShowBackingValue.Never)] public List<Vector3> PointB;

        [Output] public List<Curve> curvout;

        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.curveDictPreview.ContainsKey(id)) { pp.curveDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        int id;
        bool preview;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }

        public override object GetValue(NodePort port)
        {
            PointA = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("PointA", this.PointA));
            PointB = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("PointB", this.PointB));

           
            int longestList = Mathf.Max(PointA.Count, PointB.Count); // Find the longest list length
            //Fill in the missing elements in input lists by repeating the last element
            while (PointA.Count < longestList)
            {
                PointA.Add(PointA[PointA.Count - 1]);
            }
            while (PointB.Count < longestList)
            {
                PointB.Add(PointB[PointB.Count - 1]);
            }
            List<Curve> lines= new List<Curve>();
            for (int i = 0; i < longestList; i++)
            {
                Vector3[] crtlPoint = new Vector3[2];
                crtlPoint[0] = PointA[i];
                crtlPoint[1] = PointB[i];
                Curve recti = CreateInstance<Curve>();
                recti.type = 0;
                recti.ControlPoints = crtlPoint;

                lines.Add(recti);
            }
            return lines ;
        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);


        }
    }
    [CustomNodeEditor(typeof(LineAB))]
    public class LineABEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            LineAB cc = (LineAB)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}