using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using plugpea;
using XNode;
using XNodeEditor;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Polygon (Plane Rad.)", 4)]
    [DontFold]
    public class Polygon : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Plane> Plane;
        [Input(ShowBackingValue.Never)] public List<float> Radius;
        [Input(ShowBackingValue.Never)] public List<float> Sides;

        [Output] public List<Curve> Curve;

        [HideInInspector]
        bool preview ;

        int id;
      
        PlugPeaGraph currentGraph;

        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }
        public override object GetValue(NodePort port)
        {

            // Debug.Log(currentGraph.childIDs.Count);
            //currentGameobject = gg.currentGameobject;

            List<Plane> planes = new List<Plane>();
            List<float> radii = new List<float>();
            List<Curve> circles = new List<Curve>();
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);

            Plane = new List<Plane>() { defaultPlane };
            Radius = new List<float>() { 5f };
            Sides = new List<float>() { 5f };
            // Get the input plane and radius lists


            radii = utils.InsetMultiConnections(GetInputValues<List<float>>("Radius", Radius));

            planes = utils.InsetMultiConnections(GetInputValues<List<Plane>>("Plane", Plane));

            var sadii = utils.InsetMultiConnections(GetInputValues<List<float>>("Sides", Sides));

            if (radii == null || planes == null || sadii == null) { return null; }
            int longestList = Mathf.Max(planes.Count, radii.Count); // Find the longest list length



            //Fill in the missing elements in input lists by repeating the last element
            while (planes.Count < longestList)
            {
                planes.Add(planes[planes.Count - 1]);
            }
            while (radii.Count < longestList)
            {
                radii.Add(radii[radii.Count - 1]);
            }
            while (sadii.Count < longestList)
            {
                sadii.Add(sadii[sadii.Count - 1]);
            }
            circles.Clear();

            // Generate points on circles and create Curve instances
            for (int i = 0; i < longestList; i++)
            {
                Vector3[] pts = Curves.GeneratePointsOnCircle(planes[i], radii[i], (int)sadii[i]);
                Curve circle = CreateInstance<Curve>();
                circle.radius = radii[i];
                circle.center = planes[i];
                circle.ControlPoints = pts;
                circle.type = 0;

                circles.Add(circle);
            }

            // Remove the extra circle dictionary entries if the longest list size has decreased


            // Return the output circles list

            return circles;



        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        //}
        protected override void Init()
        {


            //-----------------------------
            base.Init();
            List<Curve> circles = new List<Curve>();
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);
        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);


        }
    }

    [CustomNodeEditor(typeof(Polygon))]
    public class PolygonEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            Polygon cc = (Polygon)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}