using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices.WindowsRuntime;
using Unity.VisualScripting;
using UnityEngine;
using XNode;
using plugpea;
using XNodeEditor;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Polyline From Points", 4)]
    [DontFold]
    public class Polyline : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Vector3> points;
        [HideInInspector]
        public bool closed;
        [HideInInspector]
        public bool Bezier;
        [HideInInspector]
        public bool preview ;
        [Output] public List<Curve> Curve;
        int id = 0;
        PlugPea pp;
        string graphName;
        GameObject currentGameobject;



        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.curveDictPreview.ContainsKey(id)) { pp.curveDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        [HideInInspector]
        public string previewName = "Preview On";
        [HideInInspector]
        public string bezierName = "Bezier";
        [HideInInspector]
        public string curveCap = "Close";

        public void closeCurve()
        {
            if (closed == true) { closed = false; curveCap = "Open"; } else { closed = true; curveCap = "Close"; }
            currentGraph.UpdateAndPropogate(this);
            

        }

        public void bezierPolyline()
        {
            if (Bezier == false) { Bezier = true; bezierName = "Bezier"; } else { Bezier = false; bezierName = "Polyline"; }
            currentGraph.UpdateAndPropogate(this);
        }

        public override string GetString()
        {
            string name = "Draw";
            if (preview == false) { name = null; }
            return name;
        }

        protected override void Init()
        {

            base.Init();
            preview = true;

        }
        public override object GetValue(NodePort port)
        {
            var gg = (PlugPeaGraph)this.graph;

            var points = GetInputValue<List<Vector3>>("points", this.points);
            bool closed = GetInputValue<bool>("closed", this.closed);
            bool bezier = GetInputValue<bool>("bezier", this.Bezier);

            if (points == null) { return null; }
            Curve pl = CreateInstance<Curve>();
            if (bezier) { pl.type = 1; } else { pl.type = 0; }
            if (closed) { pl.closed = true; } else { pl.closed = false; }


            if (closed)
            {
                pl.ControlPoints = new Vector3[points.Count + 1];

            }
            else
            {
                pl.ControlPoints = new Vector3[points.Count];
            }

            for (int i = 0; i < points.Count; i++)
            {
                pl.ControlPoints[i] = points[i];

            }
            if (closed)
            {
                pl.ControlPoints[points.Count] = pl.ControlPoints[0];
            }


            List<Curve> pls = new List<Curve>() { pl };

            return pls;
        }
        [HideInInspector]

        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }

    }
    [CustomNodeEditor(typeof(Polyline))]
    public class PolylineNodeEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {

            Polyline cc = (Polyline)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                cc.OnClickButton();
            }

            if (GUILayout.Button(cc.bezierName))
            {
                cc.bezierPolyline();
            }

            if (GUILayout.Button(cc.curveCap))
            {
                cc.closeCurve();
            }

        }

    }
}