using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Rectangle (Plane X,Y.)", 4)]
    [DontFold]
    public class Rectangle : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Plane> Plane;
        [Input(ShowBackingValue.Never)] public List<float> X;
        [Input(ShowBackingValue.Never)] public List<float> Y;

        [Output] public List<Curve> Rect;
        int id;
        bool preview;
        PlugPeaGraph currentGraph;

        protected override void Init()
        {
            base.Init();
            List<float> X = new List<float>() { 5f };
            List<float> Y = new List<float>() { 5f };
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);
            List<Plane> Plane = new List<Plane>() { defaultPlane };
        }
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }


        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.curveDictPreview.ContainsKey(id)) { pp.curveDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override object GetValue(NodePort port)
        {
            Y = utils.InsetMultiConnections(GetInputValues<List<float>>("Y", Y));
            X= utils.InsetMultiConnections(GetInputValues<List<float>>("X", X));
            if (Plane==null || Plane.Count==0)
            {
                Plane defaultPlane = CreateInstance<Plane>();
                defaultPlane.basePoint = new Vector3(0, 0, 0);
                defaultPlane.X = new Vector3(1, 0, 0);
                defaultPlane.Y = new Vector3(0, 0, 1);
                Plane = new List<Plane>() { defaultPlane };
            }
            if (X==null ||X.Count == 0)
            {
                X = new List<float>() { 1 };
            }
            if(Y == null||Y.Count == 0)
            {
                Y = new List<float>() { 1 };
            }
            Plane = utils.InsetMultiConnections(GetInputValues<List<Plane>>("Plane", Plane));

            int longestList = Mathf.Max(Plane.Count, X.Count, Y.Count); // Find the longest list length
            //Fill in the missing elements in input lists by repeating the last element
            while (Plane.Count < longestList)
            {
                Plane.Add(Plane[Plane.Count - 1]);
            }
            while (X.Count < longestList)
            {
                X.Add(X[X.Count - 1]);
            }
            while (Y.Count < longestList)
            {
                Y.Add(Y[Y.Count - 1]);
            }


            List<Curve> rectangles = new List<Curve>();
            for(int i = 0; i < longestList; i++)
            {
               
                Curve recti = CreateInstance<Curve>();
                recti.type = 0;
                recti.ControlPoints = Curves.GenerateRectangleOnPlane(Plane[i], X[i], Y[i]);
                recti.closed = true;
                rectangles.Add(recti);
            }

            return rectangles;
        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);

        }
    }

    [CustomNodeEditor(typeof(Rectangle))]
    public class RectangleEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            Rectangle cc = (Rectangle)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}
