using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Construct (Mesh)", 4)]
    [DontFold]
    public class ConstructMesh : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Vector3> Vertices;
        [Input(ShowBackingValue.Never)] public List<int> Triangles;

        [SerializeField]
        [HideInInspector]
        public bool preview;
        [Output] public List<Mesh> outMesh;
        [HideInInspector]
        public int id;
        [HideInInspector]
        public PlugPeaGraph currentGraph;
        // Start is called before the first frame update
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.UpdateNode(this);

            if (previewName == "Preview On")
            {
                preview = true;
                if (!currentGraph.previewIds.Contains(id))
                {
                    currentGraph.previewIds.Add(id);
                }
            }
            else
            {
                if (currentGraph.previewIds.Contains(id))
                {
                    currentGraph.previewIds.Remove(id);
                }
            }

        }
        //fe
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.meshDictPreviewBool.ContainsKey(id)) { pp.meshDictPreviewBool.Remove(id); }
            if (currentGraph.previewIds.Contains(id))
            {
                currentGraph.previewIds.Remove(id);
            }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override string GetString()
        {
            return "Draw";
        }

        public override object GetValue(NodePort port)
        {

            Vertices = GetInputValue<List<Vector3>>("Vertices", this.Vertices);
            Triangles = GetInputValue<List<int>>("Triangles", this.Triangles);
            int[] tritoArray = new int[Triangles.Count];
            for (int i = 0; i < tritoArray.Length; i++)
            {
                tritoArray[i] = Triangles[i];
            }
            Mesh oMehs = new Mesh();
            oMehs.SetVertices(Vertices);
            oMehs.triangles = tritoArray;
            oMehs.RecalculateNormals();

            return new List<Mesh>() { oMehs };
        }

        [HideInInspector]
        public string previewName = "Preview On";


    }
   
    [CustomNodeEditor(typeof(ConstructMesh))]
    public class constructMeshEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            ConstructMesh cc = (ConstructMesh)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                cc.currentGraph.PreviewOnOff( cc.id , ref cc.previewName,ref cc.preview);
            }
        }

    }
}
