using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Cube (Plane X,Y.)", 4)]
    [DontFold]
    public class CubeCenter : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Plane> Plane;
        [Input(ShowBackingValue.Never)] public List<float> X;
        [Input(ShowBackingValue.Never)] public List<float> Y;
        [Input(ShowBackingValue.Never)] public List<float> Z;


        [Output] public List<Mesh> Meshes;
        int id;
        bool preview;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;


            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }


        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.meshDictPreview.ContainsKey(id)) { pp.meshDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override object GetValue(NodePort port)
        {
            List<float> Ye = new List<float>();
            List<float> Xe = new List<float>();
            List<float> Ze = new List<float>();
            List<Plane> Planee = new List<Plane>();

            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);
            Plane = new List<Plane>() { defaultPlane };
            Y = new List<float>() { 5f };
            X = new List<float>() { 5f };
            Z = new List<float>() { 5f };
            Ye = utils.InsetMultiConnections(GetInputValues<List<float>>("Y", Y));
            Xe = utils.InsetMultiConnections(GetInputValues<List<float>>("X", X));
            Ze = utils.InsetMultiConnections(GetInputValues<List<float>>("Z", Z));


            Planee = utils.InsetMultiConnections(GetInputValues<List<Plane>>("Plane", Plane));


            int longestList = Mathf.Max(Planee.Count, Xe.Count, Ye.Count, Ze.Count); // Find the longest list length
                                                                                                     //int longestList = 10;

            //Fill in the missing elements in input lists by repeating the last element
            while (Planee.Count < longestList)
            {
                Planee.Add(Planee[Planee.Count - 1]);
            }
            while (Xe.Count < longestList)
            {
                Xe.Add(Xe[Xe.Count - 1]);
            }
            while (Ye.Count < longestList)
            {
                Ye.Add(Ye[Ye.Count - 1]);
            }
            while (Ze.Count < longestList)
            {
                Ze.Add(Ze[Ze.Count - 1]);
            }

            //Debug.Log(longestList + "Longest");
            
            List<Mesh> Meshrectangles = new List<Mesh>();
            for (int i = 0; i < longestList; i++)
            {

                Mesh recti = PMesh.GenerateCube(Xe[i], Ye[i], Ze[i], Planee[i]);

                Meshrectangles.Add(recti);
            }

            return Meshrectangles;
        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {



            PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);

            //wd

        }
    }

    [CustomNodeEditor(typeof(CubeCenter))]
    public class cubeCenterEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            CubeCenter cc = (CubeCenter)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}
