using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Extrude (Mesh)", 4)]
    [DontFold]
    public class ExtrudeCurve : BaseNode
    {
        [Input(ShowBackingValue.Never, ConnectionType.Multiple)]
        public List<Curve> Curves; // Input list of planes


        [Input(ShowBackingValue.Never ,ConnectionType.Multiple)]
        public List<Vector3> Vector; // Input list of radii


        [Output]
        public List<Mesh> OutputMesh; // Output list of circles
   
        [HideInInspector]
        bool preview;

        int id;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;

            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }
        protected override void Init()
        {
            base.Init();
            Vector = new List<Vector3>() { new Vector3(0, 10, 0) };
        }
        List<Mesh> meshes = new List<Mesh>();
        public override object GetValue(NodePort port)
        {
            List<Curve> crvs = new List<Curve>();
            List<Vector3> vectors = new List<Vector3>();

            crvs = utils.InsetMultiConnections(GetInputValues<List<Curve>>("Curves", this.Curves));

            vectors = utils.InsetMultiConnections(GetInputValues<List<Vector3>>("Vector", Vector));

            if (crvs == null ) { return null; }
            //if(vectors==null || vectors.Count == 0)
            //{
            //    vectors.Add(new Vector3(0, 10, 0));
            //}
 

            int longestList = Mathf.Max(vectors.Count, crvs.Count);
    
            while (crvs.Count < longestList)
            {
                crvs.Add(crvs[crvs.Count - 1]);
            }
            while (vectors.Count < longestList)
            {
                vectors.Add(vectors[vectors.Count - 1]);
            }
 
            meshes.Clear();
            for (int i = 0; i < longestList; i++)
            {
                Mesh m = PMesh.ExtrudeCurve(crvs[i], vectors[i]);
                meshes.Add(PMesh.JoinMeshes(m));
            }
           
            return meshes;
        }

        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.meshDictPreview.ContainsKey(id)) { pp.meshDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);

        }
    }

    [CustomNodeEditor(typeof(ExtrudeCurve))]
    public class ExtrudeEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            ExtrudeCurve cc = (ExtrudeCurve)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");wwess
                cc.OnClickButton();

            }
        }

    }
}