using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using System.Net.Http.Headers;
using plugpea;
using XNodeEditor;
using System.Runtime.InteropServices;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Polyline To (Mesh)", 5)]
    [DontFold]
    public class MeshFromPolyline : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Curve> curves;
        // Start is called before the first frame update
        [Input(ShowBackingValue.Never)] public List<float> Resolution;
        [Output] public List<Mesh> meshOut;

        [HideInInspector]
        List<Mesh> meshOutResult;
        [HideInInspector]
        private int id;
        [HideInInspector]
        public bool preview = true;

        PlugPea pp;
        string graphName;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;
            var pp = currentGraph.getPlugPeaViaGUID();
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);

            currentGraph.UpdateNode(this);
        }
        protected override void Init()
        {
            //Create a GUIStyle for the slider
            base.Init();
            Resolution = new List<float>();

        }
        public override string GetString()
        {
            return "Draw";
        }




        public override object GetValue(NodePort port)
        {
            id = GetInstanceID();
            List<Curve>[] curvess = GetInputValues<List<Curve>>("curves", this.curves);

            List<NodePort> portss = GetInputPort("curves").GetConnections();




            List<Curve> curves = new List<Curve>();
            if (curvess.Length > 0)
            {
                for (int i = 0; i < curvess.Length; i++)
                {
                    if (curvess[i] != null)
                    {
                        for (int j = 0; j < curvess[i].Count; j++)
                        {
                            if (curvess[i][j] != null)
                                curves.Add(curvess[i][j]);
                        }
                    }
                }
            }


            List<float> resolution = GetInputValue<List<float>>("Resolution", this.Resolution);



            List<Vector3[]> points = new List<Vector3[]>();
            if (curves != null)
            {

                foreach (Curve crv in curves)
                {
                    if (resolution != null)
                    {
                        if (resolution.Count == 0)
                        {
                            Vector3[] pts = crv.ControlPoints;
                            points.Add(pts);

                        }

                        else if (resolution.Count == 1)
                        {
                            Vector3[] tempPts = new Vector3[(int)resolution[0] + 1];
                            for (int i = 0; i < (int)resolution[0] + 1; i++)
                            {
                                float t = i / (float)resolution[0];
                                tempPts[i] = Curves.CurveInterpolation.GetPointOnCurve(crv, crv.type, t);
                            }
                            points.Add(tempPts);
                        }
                    }
                }
            }
            Mesh mesh = PMesh.MeshLoft(points, false);
            mesh.RecalculateNormals();
            Mesh mesh2 = PMesh.DoubleSide(mesh);
            mesh.RecalculateNormals();
            Mesh finalMesh = PMesh.JoinMeshes(mesh, mesh2);
            //  finalMesh.RecalculateNormals();
            meshOutResult = new List<Mesh>() { finalMesh };
            //meshOutResult.Add();


            return meshOutResult;

        }


        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.meshDictPreviewBool.ContainsKey(id)) { pp.meshDictPreviewBool.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
       
        [HideInInspector]
        public string previewName = "Preview On";

        public void OnClickButton()
        {

            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);

        }

    }
    [CustomNodeEditor(typeof(MeshFromPolyline))]
    public class meshfrompolylineEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            MeshFromPolyline cc = (MeshFromPolyline)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
               // Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}