using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using UnityEngine.UIElements;
using plugpea;
using XNodeEditor;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Sphere (Mesh)", 5)]
    [DontFold]
    public class MeshSphere : BaseNode
    {
        // Start is called before the first frame updatewd
        [Input(ShowBackingValue.Never)] public List<Vector3> Point;
        [Input(ShowBackingValue.Never)] public List<float> Radius;
        [Input(ShowBackingValue.Never)] public List<int> Resolution;
        [HideInInspector]
       public  bool preview = true;
        [HideInInspector]
        public string previewName = "Preview On";
        [Output] public List<Mesh> sphere;
        [HideInInspector]
       public int id;
        [HideInInspector]
        public PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            //Debug.Log("I'm enablesd");
            base.OnEnable();
            id = GetInstanceID() ;
            currentGraph = (PlugPeaGraph)this.graph;
            
            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
            //
        }
        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            currentGraph.meshDictPreview.Remove(id);


        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;

            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;

            gg.UpdateAndPropogate(this);

        }





        protected override void Init()
        {
            base.Init();
            Resolution = new List<int>() { 10 };
            Point = new List<Vector3>() { Vector3.zero };
            Radius = new List<float>() { 10f };
        }
        public override string GetString()
        {
            return "Draw";
        }
        int longestList = 0;
        public override object GetValue(NodePort port)
        {
            //find longest list 
            var Point = GetInputValue<List<Vector3>>("Point", this.Point);
            var Radius = GetInputValue<List<float>>("Radius", this.Radius);
            var Resolution = GetInputValue<List<int>>("Resolution", this.Resolution);

            longestList = Point.Count;
            if (Radius.Count > longestList)
            {
                longestList = Radius.Count;
            }
            while (Point.Count < longestList)
            {
                Point.Add(Point[Point.Count - 1]);
            }
            while (Radius.Count < longestList)
            {
                Radius.Add(Radius[Radius.Count - 1]);
            }
            List<Mesh> spheres = new List<Mesh>();
            for (int i = 0; i < longestList; i++)
            {
                
                Mesh spher = PMesh.GenerateSphere(Point[i], Radius[i], Resolution[0], Resolution[0]);
                spher.RecalculateNormals();
                spher.RecalculateTangents();
                spher.RecalculateUVDistributionMetric(0, 1f);
                spheres.Add(spher);
            }

            return spheres;

        }


        public void OnClickButton()
        {
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
            //if (currentGraph.previewIds.Contains(id))
            //{
            //    previewName = "Preview off";
            //    currentGraph.previewIds.Remove(id);
            //    PlayerPrefs.SetInt("Preview", 0);
            //    preview = false;
            //}
            //else
            //{
            //    previewName = "Preview On";
            //    currentGraph.previewIds.Add(id);
            //    PlayerPrefs.SetInt("Preview", 1);
            //    preview = true;
            //}
            //PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;

            //if (preview == false)
            //{
            //    preview = true;
            //    previewName = "Preview On";
            //    PlugPea[] components = FindObjectsOfType<PlugPea>();
            //    for (int i = 0; i < components.Length; i++)
            //    {
            //        PlugPea targetComponent = components[i];
            //        if (targetComponent.Id == currentGraph.plugpeaID)
            //        {
            //            var dict = targetComponent.meshDictPreviewBool;
            //            if (!dict.ContainsKey(id))
            //            {
            //                dict.Add(id, preview);
            //                currentGraph.UpdateNode(this);
            //            }
            //            else
            //            {
            //                dict[id] = preview;
            //            }
            //        }
            //    }
            //}
            //else
            //{
            //    preview = false;
            //    previewName = "Preview Off";
            //    PlugPea[] components = FindObjectsOfType<PlugPea>();
            //    for (int i = 0; i < components.Length; i++)
            //    {
            //        PlugPea targetComponent = components[i];
            //        if (targetComponent.Id == currentGraph.plugpeaID)
            //        {
            //            var dict = targetComponent.meshDictPreviewBool;
            //            if (!dict.ContainsKey(id))
            //            {
            //                dict.Add(id, preview);
            //                currentGraph.UpdateNode(this);
            //            }
            //            else
            //            {
            //                dict[id] = preview;
            //            }
            //        }
            //    }

            //}wf



        }


        
    }//e
    [CustomNodeEditor(typeof(MeshSphere))]
    public class meshSphereEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            MeshSphere cc = (MeshSphere)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                cc.OnClickButton();

            }
        }

    }
}