using System.Collections.Generic;
using UnityEngine;
using XNode;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using XNodeEditor;
using plugpea;
using System.Numerics;
using Vector3 = UnityEngine.Vector3;

namespace PlugPeaNodes
{
    [CreateNodeMenu("PlugPea/Mesh Surface (Plane X,Y.)", 4)]
    [DontFold]
    public class MeshSurface : BaseNode
    {
        [Input(ShowBackingValue.Never)] public List<Plane> Plane;
        [Input(ShowBackingValue.Never)] public List<float> X;
        [Input(ShowBackingValue.Never)] public List<float> Y;
        [Input(ShowBackingValue.Never)] public List<float> ResX;
        [Input(ShowBackingValue.Never)] public List<float> ResY;

        [Output] public List<Mesh> Meshes;
        int id;
        bool preview;
        PlugPeaGraph currentGraph;
        protected override void OnEnable()
        {
            base.OnEnable();
            id = GetInstanceID();
            currentGraph = (PlugPeaGraph)this.graph;


            currentGraph.PreviewOnOffEnable(id, ref previewName, ref preview);
            currentGraph.UpdateNode(this);
        }


        private void OnDestroy()
        {

            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            PlugPea pp = gg.getPlugPeaViaGUID();
            if (pp.meshDictPreview.ContainsKey(id)) { pp.meshDictPreview.Remove(id); }
        }
        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            PlugPeaGraph gg = (PlugPeaGraph)this.graph;
            gg.UpdateAndPropogate(this);

        }
        public override object GetValue(NodePort port)
        {
            List<float> Ye = new List<float>();
            List<float> Xe = new List<float>();
            List<Plane> Planee = new List<Plane>();
            List<float> ResYe = new List<float>();
            List<float> ResXe = new List<float>();
            Plane defaultPlane = CreateInstance<Plane>();
            defaultPlane.basePoint = new Vector3(0, 0, 0);
            defaultPlane.X = new Vector3(1, 0, 0);
            defaultPlane.Y = new Vector3(0, 0, 1);
            Plane = new List<Plane>() { defaultPlane };
            Y = new List<float>() { 5f };
            X = new List<float>() { 5f };
            ResX = new List<float>() { 5f };
            ResY = new List<float>() { 5f };
            Ye = utils.InsetMultiConnections(GetInputValues<List<float>>("Y", Y));
            Xe = utils.InsetMultiConnections(GetInputValues<List<float>>("X", X));

            ResYe = utils.InsetMultiConnections(GetInputValues<List<float>>("ResY", ResY));
            ResXe = utils.InsetMultiConnections(GetInputValues<List<float>>("ResX", ResX));

            Planee = utils.InsetMultiConnections(GetInputValues<List<Plane>>("Plane", Plane));


            int longestList = Mathf.Max(Planee.Count, Xe.Count, Ye.Count,ResYe.Count, ResXe.Count ); // Find the longest list length
            //int longestList = 10;
            
            //Fill in the missing elements in input lists by repeating the last element
            while (Planee.Count < longestList)
            {
                Planee.Add(Planee[Planee.Count - 1]);
            }
            while (Xe.Count < longestList)
            {
                Xe.Add(Xe[Xe.Count - 1]);
            }
            while (Ye.Count < longestList)
            {
                Ye.Add(Ye[Ye.Count - 1]);
            }
            while (ResXe.Count < longestList)
            {
                ResXe.Add(ResXe[ResXe.Count - 1]);
            }
            while (ResYe.Count < longestList)
            {
                ResYe.Add(ResYe[ResYe.Count - 1]);
            }
            //Debug.Log(longestList + "Longest");
            List<Mesh> Meshrectangles = new List<Mesh>();
            for (int i = 0; i < longestList; i++)
            {

                Mesh recti = PMesh.GenerateRectangularMesh(Xe[i], Ye[i],(int) ResXe[i],(int) ResYe[i], Planee[i]);

                Meshrectangles.Add(recti);
            }

            return Meshrectangles;
        }
        public override string GetString()
        {
            return "Draw";
        }
        [HideInInspector]
        public string previewName = "Preview On";
        public void OnClickButton()
        {
            PlugPeaGraph currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.PreviewOnOff(id, ref previewName, ref preview);
        }
    }

    [CustomNodeEditor(typeof(MeshSurface))]
    public class MeshsurfaceEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            MeshSurface cc = (MeshSurface)target;
            base.OnBodyGUI();

            if (GUILayout.Button(cc.previewName))
            {
                //Debug.Log("pressed");
                cc.OnClickButton();


            }
        }

    }
}
