using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
using plugpea;


namespace PlugPeaNodes
{
    [CreateNodeMenu("Input/Mesh", 2)]
    [DontFold]
    [NodeTint(15, 15, 15)]
    public class meshInput : BaseNode
    {
        [Output(ShowBackingValue.Never)] public List<Curve> mesh;

        [HideInInspector]
        public object input;

        [HideInInspector]
        public string textExport = "No Value";
        public override string GetString()
        {
            return "Draw";
        }

        GameObject currentGameobject;
        PlugPeaGraph currentGraph;
        [HideInInspector]
        public int id= 0;
        private const string PlayerPrefsKeyPrefix = "MeshInputID_";
        protected override void OnEnable()
        {
            base.OnEnable();
            if (id == 0)
            {
                string playerPrefsKey = PlayerPrefsKeyPrefix + GetInstanceID();
                id = PlayerPrefs.GetInt(playerPrefsKey, GetInstanceID());
            }
            currentGraph = (PlugPeaGraph)this.graph;

            currentGraph.UpdateNode((BaseNode)this);

        }

        protected override void Init()
        {
            base.Init();
        }
        public override object GetValue(NodePort port)
        {
            textExport = "No Value";


            var pp = currentGraph.getPlugPeaViaGUID();
            var currentGameobject = pp.gameObject;
            List<Mesh> curves = new List<Mesh>();
            MeshIn[] meshIns = currentGameobject.GetComponents<MeshIn>();
            int similarities = 0;
            foreach (MeshIn meshIn in meshIns)
            {
                if (meshIn.id == id)
                {
                    similarities += 1;
                    if (meshIn.mesh != null)
                    {
                        curves = meshIn.mesh.curveout;
                        meshIn.addToList();

                    }
                }


            }
            if (similarities == 0)
            {


                MeshIn crvIn = currentGameobject.AddComponent<MeshIn>();
                crvIn.hostNode = this;
                crvIn.id = id;


            }



            return curves;




        }
        private void OnDestroy()
        {
            string playerPrefsKey = PlayerPrefsKeyPrefix + GetInstanceID();
            PlayerPrefs.SetInt(playerPrefsKey, id);
            PlayerPrefs.Save();
            var pp = currentGraph.getPlugPeaViaGUID();
            var currentGameobject = pp.gameObject;
            CurveIn[] ccs = currentGameobject.GetComponents<CurveIn>();
            foreach (CurveIn comp in ccs)
            {
                if (comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }

        }


        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);

            currentGraph.UpdateAndPropogate(this);


        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            currentGraph.UpdateAndPropogate(this);

        }


    }

}
