using plugpea;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using XNode;
namespace PlugPeaNodes
{
    [CreateNodeMenu("Input/Number", 2)]
    [DontFold]
    [NodeTint(15, 15, 15)]
    public class numberInput : BaseNode
    {
        [Output(ShowBackingValue.Never)] public List<float> numbers;

        [HideInInspector]
        public object input;

        [HideInInspector]
        public string textExport = "No Value";
        public override string GetString()
        {
            return "Draw";
        }
        
        GameObject currentGameobject;
        PlugPeaGraph currentGraph;
        [HideInInspector]
        public int id = 0;
        private string guidKey;
        private string idKey;

        protected override void OnEnable()
        {
            base.OnEnable();
            string guid = PlayerPrefs.GetString(guidKey, "");
            if (string.IsNullOrEmpty(guid))
            {
                guid = System.Guid.NewGuid().ToString();
                PlayerPrefs.SetString(guidKey, guid);
            }

            // Use the Guid to form the key for this node's ID.
            idKey = "ID_" + guid;

            id = PlayerPrefs.GetInt(idKey, 0);  // Load the value from PlayerPrefs using idKey.

            if (id == 0)
            {
                id = GetInstanceID();
                PlayerPrefs.SetInt(idKey, id);  // Save the new id to PlayerPrefs with the idKey.
                PlayerPrefs.Save();  // You need to call Save for the change to be persisted.
            }
            currentGraph = (PlugPeaGraph)this.graph;

            currentGraph.UpdateNode((BaseNode)this);

        }

        protected override void Init()
        {
            base.Init();
        }
        public override object GetValue(NodePort port)
        {
            textExport = "No Value";


            var pp = currentGraph.getPlugPeaViaGUID();
            var currentGameobject = pp.gameObject;
            List<float> curves = new List<float>();
            NumberIn[] meshIns = currentGameobject.GetComponents<NumberIn>();
            int similarities = 0;
            foreach (NumberIn meshIn in meshIns)
            { 
                if (meshIn.id == id)
                {
                    similarities += 1;
                    if (meshIn.numb != null)
                    {
                        curves = meshIn.numb.curveout;
                        meshIn.addToList();

                    }
                }


            }
            if (similarities == 0)
            {


                NumberIn crvIn = currentGameobject.AddComponent<NumberIn>();
                crvIn.hostNode = this;
                crvIn.id = id;

            }



            return curves;




        }
        private void OnDestroy()
        {
            PlayerPrefs.DeleteKey(idKey);  // Reset the id when the object is destroyed.
            PlayerPrefs.Save();

            var pp = currentGraph.getPlugPeaViaGUID();
            var currentGameobject = pp.gameObject;
            NumberIn[] ccs = currentGameobject.GetComponents<NumberIn>();
            foreach (NumberIn comp in ccs)
            {
                if (comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }

        }


        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);

            currentGraph.UpdateAndPropogate(this);


        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            currentGraph.UpdateAndPropogate(this);

        }

    }
}
