using UnityEngine;  
using XNode;
using XNodeEditor;
using System.Collections.Generic;
using plugpea;

namespace PlugPeaNodes
{
    [CreateNodeMenu("Output", 3)]
    [DontFold]
    [NodeTint(15,15,15)]
    public class outputconnection : BaseNode
    {
        [Input(ShowBackingValue.Never)] public object thingsToExport;

        [HideInInspector]
        public string textExport = "No Value";
        public override string GetString()
        {
            return "Draw";
        }

        GameObject currentGameobject;
        PlugPeaGraph currentGraph;
        [HideInInspector]
        public int id = 0;
        private string guidKey;
        private string idKey;
        protected override void OnEnable()
        {
            base.OnEnable();
            string guid = PlayerPrefs.GetString(guidKey, "");
            if (string.IsNullOrEmpty(guid))
            {
                guid = System.Guid.NewGuid().ToString();
                PlayerPrefs.SetString(guidKey, guid);
            }
            idKey = "ID_" + guid;

            id = PlayerPrefs.GetInt(idKey, 0);  // Load the value from PlayerPrefs using idKey.

            if (id == 0)
            {
                id = GetInstanceID();
                PlayerPrefs.SetInt(idKey, id);  // Save the new id to PlayerPrefs with the idKey.
                PlayerPrefs.Save();  // You need to call Save for the change to be persisted.
            }
            currentGraph = (PlugPeaGraph)this.graph;
            currentGraph.UpdateNode((BaseNode)this);

        }

        protected override void Init()
        {
            base.Init();
        }
        public override object GetValue(NodePort port)
        {
            textExport = "No Value";


            object input = GetInputValue("thingsToExport", this.thingsToExport);
          
            if (input != null)
            {
                var pp = currentGraph.getPlugPeaViaGUID();
                var currentGameobject = pp.gameObject;
      
                    
                if (input is List<Curve> curveOutput)
                {
                  
                    CurveOut[] ccs = currentGameobject.GetComponents<CurveOut>();
                    if (ccs == null || ccs.Length == 0)
                    {
                        var comp = currentGameobject.AddComponent<CurveOut>();
                        comp.id = id;
                        comp.curveout.Clear();
                        for (int i = 0; i < curveOutput.Count; i++)
                        {
                            comp.curveout.Add(curveOutput[i]);
                        }
                        textExport = comp.curveout.Count+" Curve";
                    }
                    else
                    {
                        int similarCount = 0;
                        foreach(CurveOut comp in ccs)
                        {
                            if (comp.id == id)
                            {
                                similarCount += 1;
                                comp.curveout.Clear();
                                for (int i = 0; i < curveOutput.Count; i++)
                                {
                                    comp.curveout.Add(curveOutput[i]);
                                }
                                textExport = comp.curveout.Count + " Curve";
                                comp.updateConnectedNodes();
                            }
                            
                        }
                        if (similarCount == 0)
                        {
                            var comp = currentGameobject.AddComponent<CurveOut>();
                            comp.id = id;
                            comp.curveout.Clear();
                            for (int i = 0; i < curveOutput.Count; i++)
                            {
                                comp.curveout.Add(curveOutput[i]);
                            }
                            textExport = comp.curveout.Count + " Curve";

                        }
                    }
                }
                //-------------------------------------------------------------------------------------------------------------

                if (input is List<Mesh> meshOutput)
                {

                    MeshOut[] ccs = currentGameobject.GetComponents<MeshOut>();
                    if (ccs == null || ccs.Length == 0)
                    {
                        var comp = currentGameobject.AddComponent<MeshOut>();
                        comp.id = id;
                        comp.curveout.Clear();
                        for (int i = 0; i < meshOutput.Count; i++)
                        {
                            comp.curveout.Add(meshOutput[i]);
                        }
                        textExport = comp.curveout.Count + " Mesh";
                    }
                    else
                    {
                        int similarCount = 0;
                        foreach (MeshOut comp in ccs)
                        {
                            if (comp.id == id)
                            {
                                similarCount += 1;
                                comp.curveout.Clear();
                                for (int i = 0; i < meshOutput.Count; i++)
                                {
                                    comp.curveout.Add(meshOutput[i]);
                                }
                                textExport = comp.curveout.Count + " Mesh";
                                comp.updateConnectedNodes();
                            }

                        }
                        if (similarCount == 0)
                        {
                            var comp = currentGameobject.AddComponent<MeshOut>();
                            comp.id = id;
                            comp.curveout.Clear();
                            for (int i = 0; i < meshOutput.Count; i++)
                            {
                                comp.curveout.Add(meshOutput[i]);
                            }
                            textExport = comp.curveout.Count + " Mesh";

                        }
                    }
                }
                //-----------------------------------------------------------------------------------------
                if (input is List<Plane> planeOutput)
                {

                    PlaneOut[] ccs = currentGameobject.GetComponents<PlaneOut>();
                    if (ccs == null || ccs.Length == 0)
                    {
                        var comp = currentGameobject.AddComponent<PlaneOut>();
                        comp.id = id;
                        comp.curveout.Clear();
                        for (int i = 0; i < planeOutput.Count; i++)
                        {
                            comp.curveout.Add(planeOutput[i]);
                        }
                        textExport = comp.curveout.Count + " Plane";
                    }
                    else
                    {
                        int similarCount = 0;
                        foreach (PlaneOut comp in ccs)
                        {
                            if (comp.id == id)
                            {
                                similarCount += 1;
                                comp.curveout.Clear();
                                for (int i = 0; i < planeOutput.Count; i++)
                                {
                                    comp.curveout.Add(planeOutput[i]);
                                }
                                textExport = comp.curveout.Count + " Plane";
                                comp.updateConnectedNodes();
                            }

                        }
                        if (similarCount == 0)
                        {
                            var comp = currentGameobject.AddComponent<PlaneOut>();
                            comp.id = id;
                            comp.curveout.Clear();
                            for (int i = 0; i < planeOutput.Count; i++)
                            {
                                comp.curveout.Add(planeOutput[i]);
                            }
                            textExport = comp.curveout.Count + " Plane";

                        }
                    }
                }

                //-------------------------------------------------------------------------------------
                if (input is List<float> numberOut)
                {

                    NumberOut[] ccs = currentGameobject.GetComponents<NumberOut>();
                    if (ccs == null || ccs.Length == 0)
                    {
                        var comp = currentGameobject.AddComponent<NumberOut>();
                        comp.id = id;
                        comp.curveout.Clear();
                        for (int i = 0; i < numberOut.Count; i++)
                        {
                            comp.curveout.Add(numberOut[i]);
                        }
                        textExport = comp.curveout.Count + " Number";
                    }
                    else
                    {
                        int similarCount = 0;
                        foreach (NumberOut comp in ccs)
                        {
                            if (comp.id == id)
                            {
                                similarCount += 1;
                                comp.curveout.Clear();
                                for (int i = 0; i < numberOut.Count; i++)
                                {
                                    comp.curveout.Add(numberOut[i]);
                                }
                                textExport = comp.curveout.Count + " Number";
                                comp.updateConnectedNodes();
                            }

                        }
                        if (similarCount == 0)
                        {
                            var comp = currentGameobject.AddComponent<NumberOut>();
                            comp.id = id;
                            comp.curveout.Clear();
                            for (int i = 0; i < numberOut.Count; i++)
                            {
                                comp.curveout.Add(numberOut[i]);
                            }
                            textExport = comp.curveout.Count + " Number";

                        }
                    }
                }

                //______________________________________________________________________________________

                if (input is List<Vector3> pOut)
                {

                    PointOut[] ccs = currentGameobject.GetComponents<PointOut>();
                    if (ccs == null || ccs.Length == 0)
                    {
                        var comp = currentGameobject.AddComponent<PointOut>();
                        comp.id = id;
                        comp.curveout.Clear();
                        for (int i = 0; i < pOut.Count; i++)
                        {
                            comp.curveout.Add(pOut[i]);
                        }
                        textExport = comp.curveout.Count + " Point";
                    }
                    else
                    {
                        int similarCount = 0;
                        foreach (PointOut comp in ccs)
                        {
                            if (comp.id == id)
                            {
                                similarCount += 1;
                                comp.curveout.Clear();
                                for (int i = 0; i < pOut.Count; i++)
                                {
                                    comp.curveout.Add(pOut[i]);
                                }
                                textExport = comp.curveout.Count + " Point";
                                comp.updateConnectedNodes();
                            }

                        }
                        if (similarCount == 0)
                        {
                            var comp = currentGameobject.AddComponent<PointOut>();
                            comp.id = id;
                            comp.curveout.Clear();
                            for (int i = 0; i < pOut.Count; i++)
                            {
                                comp.curveout.Add(pOut[i]);
                            }
                            textExport = comp.curveout.Count + " Point";

                        }
                    }
                }
                //----------------------------------------------------------------------

            }
            return null;
        }
        private void OnDestroy()
        {
            PlayerPrefs.DeleteKey(idKey);  // Reset the id when the object is destroyed.
            PlayerPrefs.Save();
            var pp = currentGraph.getPlugPeaViaGUID();
            var currentGameobject = pp.gameObject;
            CurveOut[] ccs = currentGameobject.GetComponents<CurveOut>();
            
            foreach(CurveOut comp in ccs)
            {
                if(comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }
            MeshOut[] ccss = currentGameobject.GetComponents<MeshOut>();
            foreach (MeshOut comp in ccss)
            {
                if (comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }

            PlaneIn[] planeIn = currentGameobject.GetComponents<PlaneIn>();
            foreach (PlaneIn comp in planeIn)
            {
                if (comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }

            NumberIn[] numberIn = currentGameobject.GetComponents<NumberIn>();
            foreach (PlaneIn comp in planeIn)
            {
                if (comp.id == id)
                {
                    DestroyImmediate(comp);
                }
            }
        }


        public override void OnCreateConnection(NodePort from, NodePort to)
        {
            base.OnCreateConnection(from, to);

            currentGraph.UpdateAndPropogate(this);


        }

        public override void OnRemoveConnection(NodePort port)
        {
            base.OnRemoveConnection(port);
            currentGraph.UpdateAndPropogate(this);

        }


    }

    [CustomNodeEditor(typeof(outputconnection))]
    public class MultiTypeNodeEditor : NodeEditor
    {
        public override void OnBodyGUI()
        {
            base.OnBodyGUI();
            outputconnection node = target as outputconnection;

            NodeEditorGUILayout.PortField(new GUIContent("Input"), node.GetPort("thingsToExport"));

            GUILayout.Label(node.textExport);
            
        }
    }
}