using System.Collections.Generic;
using UnityEngine;
namespace plugpea
{
    [RequireComponent(typeof(LineRenderer))]
    [ExecuteInEditMode]
    public class MoveLineRendererWithTransform : MonoBehaviour
    {
        private LineRenderer lineRenderer;
        private Transform cachedTransform;

        public Vector3[] initialPositions;
        Vector3 previousPos;
        Vector3 previousScale;
        Quaternion previousRot;

        private void OnEnable()
        {
            lineRenderer = GetComponent<LineRenderer>();
            initialPositions = new Vector3[lineRenderer.positionCount];
            if (lineRenderer.positionCount > 0)
            {
                for (int i = 0; i < lineRenderer.positionCount; i++)
                {
                    initialPositions[i] = (lineRenderer.GetPosition(i) - transform.position);
                }
            }
            else
            {
                Debug.LogError("LineRenderer must have at least one point!");
            }

            previousPos = transform.position;
            previousScale = transform.localScale;
            previousRot = transform.rotation;

        }


        private void Update()
        {
            // Update the positions of the LineRenderer to match the current transform
            if (previousPos != transform.position || previousRot != transform.rotation || previousScale != transform.localScale)
            {
                previousPos = transform.position;
                previousRot = transform.rotation;
                previousScale = transform.localScale;
                for (int i = 0; i < lineRenderer.positionCount; i++)
                {
                    Vector3 relativePos = transform.TransformPoint(initialPositions[i]);
                    lineRenderer.SetPosition(i, relativePos);
                }
            }
        }

    }
}