using UnityEngine;
using System.Collections.Generic;
using System.Linq;
using System;
using PlugPeaNodes;
using Plane = PlugPeaNodes.Plane;
using System.Numerics;
using Matrix4x4 = UnityEngine.Matrix4x4;
using Vector3 = UnityEngine.Vector3;

namespace plugpea
{
    public class bezierCurve
    {
        Vector3[] pts;
    }
    public static class utils
    {


        public static List<T> InsetMultiConnections<T>(List<T>[] inputObject)
        {
            try
            {
                List<T> outObject = new List<T>();
                for (int i = 0; i < inputObject.Length; i++)
                {
                    for (int j = 0; j < inputObject[i].Count; j++)
                    {
                        outObject.Add(inputObject[i][j]);
                    }
                }
                return outObject;
            }
            catch
            {
                return null;
            }


        }

        private static readonly System.Random random = new System.Random();

        public static int GenerateRandomID()
        {
            return random.Next(10000, 100000);
        }

        public static int GetLongestListLength<T>(params List<T>[] lists)
        {
            int longestLength = 0;


            foreach (List<T> list in lists)
            {
                if (list != null)
                {
                    int currentLength = list.Count;
                    if (currentLength > longestLength)
                    {
                        longestLength = currentLength;
                    }
                }

            }


            // Fill shorter lists with their latest item
            foreach (List<T> list in lists)
            {
                int currentLength;
                try
                {
                    currentLength = list.Count;
                }
                catch { currentLength = 0; }
                if (currentLength < longestLength && currentLength > 0)
                {
                    T latestItem = list[currentLength - 1];
                    for (int i = currentLength; i < longestLength; i++)
                    {
                        list.Add(latestItem);
                    }
                }
            }

            return longestLength;
        }
        public static float Remap(float value, float fromMin, float fromMax, float toMin, float toMax)
        {
            // Clamp the value to the original range
            value = Mathf.Clamp(value, fromMin, fromMax);

            // Calculate the normalized position of the value in the original range (between 0 and 1)
            float normalizedValue = (value - fromMin) / (fromMax - fromMin);

            // Map the normalized value to the new range
            float remappedValue = toMin + normalizedValue * (toMax - toMin);

            return remappedValue;
        }
        public static Vector3[] GenerateCirclePoints(int numPoints, float radius, Vector3 center)
        {
            Vector3[] points = new Vector3[numPoints];
            float angleIncrement = (Mathf.PI * 2f) / numPoints;

            for (int i = 0; i < numPoints; i++)
            {
                float angle = i * angleIncrement;
                float x = center.x + radius * Mathf.Cos(angle);
                float y = center.y + 0f; // Change this if you want points on a different plane (e.g., y = center.y + radius * Mathf.Sin(angle) for a 2D circle)
                float z = center.z + radius * Mathf.Sin(angle);

                points[i] = new Vector3(x, y, z);
            }

            return points;
        }
        public static Mesh CombineMeshes(Mesh mesh1, Mesh mesh2)
        {
            Mesh combinedMesh = new Mesh();

            // Combine the vertices, triangles, and UVs of mesh1 and mesh2 into the new mesh
            CombineInstance[] combineInstances = new CombineInstance[2];
            combineInstances[0].mesh = mesh1;
            combineInstances[0].transform = Matrix4x4.identity;
            combineInstances[1].mesh = mesh2;
            combineInstances[1].transform = Matrix4x4.identity;
            combinedMesh.CombineMeshes(combineInstances);

            return combinedMesh;
        }


        public static Vector3[] generateBezier(Transform[] controlPoints, int resolution, bool closed)
        {
            if (closed)
            {
                Array.Resize(ref controlPoints, controlPoints.Length + 1);
                controlPoints[controlPoints.Length - 1] = controlPoints[0];
            }
            Vector3[] samplePts = new Vector3[resolution + 1];
            for (int resCount = 0; resCount < resolution + 1; resCount++)
            {

                Vector3[] pts = new Vector3[controlPoints.Length];
                for (int i = 0; i < controlPoints.Length; i++)
                {
                    pts[i] = controlPoints[i].position;
                }

                samplePts[resCount] = Curves.bezier.Bezier(pts, (float)resCount / (float)resolution);
            }
            return samplePts;

        }



        public static Vector3[,] ReverseCurves(Vector3[,] curves, int resolution, bool useBezier)
        {
            // Create a new array to hold the reversed curves
            Vector3[,] reversedCurves = new Vector3[curves.GetLength(0), curves.GetLength(1)];

            // Loop through each curve in the input array
            for (int i = 0; i < curves.GetLength(0); i++)
            {
                // Reverse the order of the points in the curve
                Vector3[] reversedPoints = new Vector3[curves.GetLength(1)];
                for (int j = 0; j < curves.GetLength(1); j++)
                {
                    reversedPoints[j] = curves[i, curves.GetLength(1) - 1 - j];
                }

                // Create a new array to hold the reversed curve with the specified resolution
                Vector3[] reversedCurve = new Vector3[resolution];

                // Loop through each point in the reversed curve
                for (int j = 0; j < resolution; j++)
                {
                    // Calculate the index of the point in the original curve based on the resolution
                    float t = (float)j / (resolution - 1);
                    int index = Mathf.RoundToInt((1 - t) * (reversedPoints.Length - 1));

                    // Get the point from the original curve
                    Vector3 point = reversedPoints[index];

                    // Interpolate the point using Bezier or linear interpolation, if specified
                    if (useBezier && index > 0 && index < reversedPoints.Length - 1)
                    {
                        //// Use cubic Bezier interpolation
                        //int p0Index = Mathf.Max(0, index - 1);
                        //int p1Index = index;
                        //int p2Index = Mathf.Min(reversedPoints.Length - 1, index + 1);
                        //int p3Index = Mathf.Min(reversedPoints.Length - 1, index + 2);
                        //float t2 = (t * (reversedPoints.Length - 1) - p1Index) / (p2Index - p0Index);
                        //Vector3 p0 = reversedPoints[p0Index];
                        //Vector3 p1 = reversedPoints[p1Index];
                        //Vector3 p2 = reversedPoints[p2Index];
                        //Vector3 p3 = reversedPoints[p3Index];
                        //point = CubicBezier(p0, p1, p2, p3, t2);
                    }
                    else if (!useBezier && index > 0)
                    {
                        // Use linear interpolation
                        Vector3 prevPoint = reversedPoints[index - 1];
                        point = Vector3.Lerp(prevPoint, point, (1 - t) * (reversedPoints.Length - 1) - index);

                    }

                    // Add the interpolated point to the reversed curve array
                    reversedCurve[j] = point;
                }

                // Add the reversed curve to the output array
                for (int j = 0; j < resolution; j++)
                {
                    reversedCurves[i, j] = reversedCurve[j];
                }
            }

            // Return the array of reversed curves
            return reversedCurves;
        }



        public static Vector3[,] InterpolateCurves(Vector3[,] curves, int numInterpolations)
        {
            int numCurves = curves.GetLength(0);
            int numPointsPerCurve = curves.GetLength(1);
            Vector3[,] interpolatedCurves = new Vector3[numCurves * numInterpolations, numPointsPerCurve];

            for (int i = 0; i < numCurves; i++)
            {
                for (int j = 0; j < numPointsPerCurve; j++)
                {
                    interpolatedCurves[i * numInterpolations, j] = curves[i, numPointsPerCurve - 1 - j];
                }

                for (int j = 1; j < numInterpolations; j++)
                {
                    float t = (float)j / (float)(numInterpolations - 1);

                    for (int k = 1; k < numPointsPerCurve - 1; k++)
                    {
                        Vector3 p0 = curves[i, k + 1];
                        Vector3 p1 = curves[i, k];
                        Vector3 p2 = curves[i, k - 1];

                        Vector3 tangent1 = (p1 - p0).normalized;
                        Vector3 tangent2 = (p2 - p1).normalized;

                        Vector3 smoothTangent1 = tangent1 + (tangent2 - tangent1) * 0.5f;
                        Vector3 smoothTangent2 = tangent2 + (tangent1 - tangent2) * 0.5f;

                        Vector3 smoothP1 = p1 + smoothTangent1 * (p1 - p0).magnitude * t;
                        Vector3 smoothP2 = p1 + smoothTangent2 * (p2 - p1).magnitude * (1 - t);

                        interpolatedCurves[i * numInterpolations + j, k] = Vector3.Lerp(smoothP1, smoothP2, t);
                    }

                    interpolatedCurves[i * numInterpolations + j, 0] = curves[i, 0];
                    interpolatedCurves[i * numInterpolations + j, numPointsPerCurve - 1] = curves[i, numPointsPerCurve - 1];
                }
            }

            return interpolatedCurves;
        }


        public static Vector3 childPosition(Vector3 parentPos, Vector3 childPos)
        {

            Vector3 realPos = new Vector3(parentPos.x - childPos.x, parentPos.y - childPos.y, parentPos.z - parentPos.z);
            return realPos;
        }









        //    public static Mesh FlipMesh(Mesh originalMesh, bool flipInsideOut)
        //    {
        //        // Create a new mesh to hold the flipped mesh
        //        Mesh flippedMesh = new Mesh();

        //        // Duplicate the original mesh
        //        flippedMesh.vertices = originalMesh.vertices;
        //        flippedMesh.triangles = originalMesh.triangles;

        //        // Reverse the order of the triangle indices to flip the mesh inside out
        //        if (flipInsideOut)
        //        {
        //            for (int i = 0; i < flippedMesh.subMeshCount; i++)
        //            {
        //                int[] triangles = flippedMesh.GetTriangles(i);
        //                for (int j = 0; j < triangles.Length; j += 3)
        //                {
        //                    int temp = triangles[j];
        //                    triangles[j] = triangles[j + 2];
        //                    triangles[j + 2] = temp;
        //                }
        //                flippedMesh.SetTriangles(triangles, i);
        //            }
        //        }

        //        // Flip the face normals of the mesh
        //        flippedMesh.RecalculateNormals();
        //        Vector3[] normals = flippedMesh.normals;
        //        for (int i = 0; i < normals.Length; i++)
        //        {
        //            normals[i] = -normals[i];
        //        }
        //        flippedMesh.normals = normals;

        //        // Flip the tangents of the mesh
        //        Vector4[] tangents = flippedMesh.tangents;
        //        for (int i = 0; i < tangents.Length; i++)
        //        {
        //            tangents[i].w = -tangents[i].w;
        //        }
        //        flippedMesh.tangents = tangents;

        //        // Return the flipped mesh
        //        return flippedMesh;

        //}





        public static Mesh DoubleSide(Mesh originalMesh)
        {
            // Create a new mesh to hold the double-sided mesh
            Mesh doubleSidedMesh = new Mesh();

            // Duplicate the vertices of the original mesh
            Vector3[] vertices = originalMesh.vertices;
            int numVertices = vertices.Length;
            Vector3[] newVertices = new Vector3[numVertices * 2];
            for (int i = 0; i < numVertices; i++)
            {
                newVertices[i] = vertices[i];
                newVertices[i + numVertices] = vertices[i];
            }

            // Duplicate the normals of the original mesh
            Vector3[] normals = originalMesh.normals;
            Vector3[] newNormals = new Vector3[numVertices * 2];
            for (int i = 0; i < numVertices; i++)
            {
                newNormals[i] = normals[i];
                newNormals[i + numVertices] = -normals[i];
            }

            // Create a new set of triangles that represent the mesh as if it were double-sided
            int[] triangles = originalMesh.triangles;
            int numTriangles = triangles.Length;
            int[] newTriangles = new int[numTriangles * 2];
            for (int i = 0; i < numTriangles; i += 3)
            {
                // First set of triangles
                newTriangles[i] = triangles[i];
                newTriangles[i + 1] = triangles[i + 1];
                newTriangles[i + 2] = triangles[i + 2] + numVertices;

                // Second set of triangles
                newTriangles[i + numTriangles] = triangles[i] + numVertices;
                newTriangles[i + numTriangles + 1] = triangles[i + 2] + numVertices;
                newTriangles[i + numTriangles + 2] = triangles[i + 1] + numVertices;
            }

            // Calculate the normals for the interior faces
            for (int i = 0; i < numTriangles; i += 3)
            {
                // First set of triangles
                int normalIndex = triangles[i];
                Vector3 normal = newNormals[normalIndex];
                if (Vector3.Dot(normal, newVertices[triangles[i]] - newVertices[triangles[i + 1]]) > 0)
                {
                    normal = -normal;
                }
                newNormals[normalIndex] = normal;

                normalIndex = triangles[i + 1];
                normal = newNormals[normalIndex];
                if (Vector3.Dot(normal, newVertices[triangles[i + 1]] - newVertices[triangles[i + 2]]) > 0)
                {
                    normal = -normal;
                }
                newNormals[normalIndex] = normal;

                normalIndex = triangles[i + 2] + numVertices;
                normal = newNormals[normalIndex];
                if (Vector3.Dot(normal, newVertices[triangles[i + 2] + numVertices] - newVertices[triangles[i + 1] + numVertices]) > 0)
                {
                    normal = -normal;
                }
                newNormals[normalIndex] = normal;

                // Second set of triangles



                normalIndex = triangles[i + 2] + numVertices;
                normal = newNormals[normalIndex];
                if (Vector3.Dot(normal, newVertices[triangles[i + 2] + numVertices] - newVertices[triangles[i] + numVertices]) > 0)
                {
                    normal = -normal;
                }
                newNormals[normalIndex] = normal;

                normalIndex = triangles[i + 1] + numVertices;
                normal = newNormals[normalIndex];
                if (Vector3.Dot(normal, newVertices[triangles[i + 1] + numVertices] - newVertices[triangles[i + 2] + numVertices]) > 0)
                {
                    normal = -normal;
                }
                newNormals[normalIndex] = normal;
            }

            // Assign the new vertices, normals, and triangles to the double-sided mesh
            doubleSidedMesh.vertices = newVertices;
            doubleSidedMesh.normals = newNormals;
            doubleSidedMesh.triangles = newTriangles;

            return doubleSidedMesh;
        }








    }
}










